@@ -35,7 +35,7 @@ void setup_user_data(void) {
     }
 
     debug("creating user data directory: %s", user_data);
-    if (sc_nonfatal_mkpath(user_data, 0755) < 0) {
+    if (sc_nonfatal_mkpath(user_data, 0755, -1, -1) < 0) {
         if ((errno == EROFS || errno == EACCES) && !sc_startswith(user_data, "/home/")) {
             // clear errno or it will be displayed in die()
             errno = 0;
@@ -58,7 +58,7 @@ void setup_user_xdg_runtime_dir(void) {
 
     errno = 0;
     debug("creating user XDG_RUNTIME_DIR directory: %s", xdg_runtime_dir);
-    if (sc_nonfatal_mkpath(xdg_runtime_dir, 0755) < 0) {
+    if (sc_nonfatal_mkpath(xdg_runtime_dir, 0755, -1, -1) < 0) {
         die("cannot create user XDG_RUNTIME_DIR directory: %s", xdg_runtime_dir);
     }
     // if successfully created the directory (ie, not EEXIST), then chmod it.
