@@ -21,10 +21,11 @@ package main_test
 
 import (
 	"fmt"
-	"io/ioutil"
 	"os"
 	"os/exec"
 	"path/filepath"
+	"strings"
+	"syscall"
 	"testing"
 
 	. "gopkg.in/check.v1"
@@ -75,10 +76,20 @@ apps:
   command-chain: [chain1, chain2]
  nostop:
   command: nostop
+components:
+ comp:
+  type: standard
+  hooks:
+   install:
 `)
 
 var mockClassicYaml = append([]byte("confinement: classic\n"), mockYaml...)
 
+var mockComponentYaml = []byte(`component: snapname+comp
+type: standard
+version: 1.0
+`)
+
 var mockHookYaml = []byte(`name: snapname
 version: 1.0
 hooks:
@@ -90,6 +101,12 @@ version: 1.0
 hooks:
  configure:
   command-chain: [chain1, chain2]
+components:
+ comp:
+  type: standard
+  hooks:
+   install:
+    command-chain: [chain3, chain4]
 `)
 
 var binaryTemplate = `#!/bin/sh
@@ -181,6 +198,48 @@ func (s *snapExecSuite) TestSnapExecAppIntegration(c *C) {
 	// See also https://bugs.launchpad.net/snapd/+bug/1860369
 	c.Check(execEnv, Not(testutil.Contains), "TEST_PATH=/vanilla")
 	c.Check(execEnv, testutil.Contains, "TEST_PATH=/custom")
+
+	// ensure that CUPS_SERVER is absent since we didn't mock the /var/cups dir
+	c.Check(execEnv, Not(testutil.Contains), "CUPS_SERVER=/var/cups")
+}
+
+func (s *snapExecSuite) TestSnapExecAppIntegrationCupsServerWorkaround(c *C) {
+	dir := c.MkDir()
+	dirs.SetRootDir(dir)
+	snaptest.MockSnap(c, string(mockYaml), &snap.SideInfo{
+		Revision: snap.R("42"),
+	})
+
+	// mock the /var/cups dir is a bind-mount
+	restore := snapExec.MockSyscallStat(func(p string, st *syscall.Stat_t) error {
+		if strings.HasSuffix(p, "/var/cups/") {
+			st.Dev = 2
+		} else {
+			st.Dev = 1
+		}
+		return nil
+	})
+	defer restore()
+
+	execArgv0 := ""
+	execArgs := []string{}
+	execEnv := []string{}
+	restore = snapExec.MockSyscallExec(func(argv0 string, argv []string, env []string) error {
+		execArgv0 = argv0
+		execArgs = argv
+		execEnv = env
+		return nil
+	})
+	defer restore()
+
+	// launch and verify its run the right way
+	err := snapExec.ExecApp("snapname.app", "42", "stop", []string{"arg1", "arg2"})
+	c.Assert(err, IsNil)
+	c.Check(execArgv0, Equals, fmt.Sprintf("%s/snapname/42/stop-app", dirs.SnapMountDir))
+	c.Check(execArgs, DeepEquals, []string{execArgv0, "arg1", "arg2"})
+
+	// ensure that CUPS_SERVER is now set since we did mock the /var/cups dir
+	c.Check(execEnv, testutil.Contains, "CUPS_SERVER=/var/cups/cups.sock")
 }
 
 func (s *snapExecSuite) TestSnapExecAppCommandChainIntegration(c *C) {
@@ -320,7 +379,7 @@ func (s *snapExecSuite) TestSnapExecAppRealIntegration(c *C) {
 
 	canaryFile := filepath.Join(c.MkDir(), "canary.txt")
 	script := fmt.Sprintf("%s/snapname/42/run-app", dirs.SnapMountDir)
-	err := ioutil.WriteFile(script, []byte(fmt.Sprintf(binaryTemplate, canaryFile)), 0755)
+	err := os.WriteFile(script, []byte(fmt.Sprintf(binaryTemplate, canaryFile)), 0755)
 	c.Assert(err, IsNil)
 
 	// we can not use the real syscall.execv here because it would
@@ -620,3 +679,64 @@ func (s *snapExecSuite) TestSnapExecCompleteClassicNoReexec(c *C) {
 	c.Check(execEnv, testutil.Contains, "SNAP_DATA=/var/snap/snapname/42")
 	c.Check(execEnv, testutil.Contains, "TMPDIR=/var/tmp99")
 }
+
+func (s *snapExecSuite) TestSnapExecComponentHookIntegration(c *C) {
+	dirs.SetRootDir(c.MkDir())
+	snapInfo := snaptest.MockSnap(c, string(mockYaml), &snap.SideInfo{
+		Revision: snap.R(42),
+	})
+	snaptest.MockComponentCurrent(c, string(mockComponentYaml), snapInfo, snap.ComponentSideInfo{
+		Revision: snap.R(21),
+	})
+
+	execArgv0 := ""
+	execArgs := []string{}
+	restore := snapExec.MockSyscallExec(func(argv0 string, argv []string, env []string) error {
+		execArgv0 = argv0
+		execArgs = argv
+		return nil
+	})
+	defer restore()
+
+	hookPath := filepath.Join(dirs.SnapMountDir, "/snapname/components/mnt/comp/21/meta/hooks/install")
+
+	err := snapExec.ExecHook("snapname+comp", "42", "install")
+	c.Assert(err, IsNil)
+	c.Check(execArgv0, Equals, hookPath)
+	c.Check(execArgs, DeepEquals, []string{execArgv0})
+}
+
+func (s *snapExecSuite) TestSnapExecComponentHookCommandChainIntegration(c *C) {
+	dirs.SetRootDir(c.MkDir())
+	snapInfo := snaptest.MockSnap(c, string(mockHookCommandChainYaml), &snap.SideInfo{
+		Revision: snap.R(42),
+	})
+	snaptest.MockComponentCurrent(c, string(mockComponentYaml), snapInfo, snap.ComponentSideInfo{
+		Revision: snap.R(21),
+	})
+
+	execArgv0 := ""
+	execArgs := []string{}
+	restore := snapExec.MockSyscallExec(func(argv0 string, argv []string, env []string) error {
+		execArgv0 = argv0
+		execArgs = argv
+		return nil
+	})
+	defer restore()
+
+	chain3Path := filepath.Join(dirs.SnapMountDir, "/snapname/components/mnt/comp/21/chain3")
+	chain4Path := filepath.Join(dirs.SnapMountDir, "/snapname/components/mnt/comp/21/chain4")
+	hookPath := filepath.Join(dirs.SnapMountDir, "/snapname/components/mnt/comp/21/meta/hooks/install")
+
+	err := snapExec.ExecHook("snapname+comp", "42", "install")
+	c.Assert(err, IsNil)
+	c.Check(execArgv0, Equals, chain3Path)
+	c.Check(execArgs, DeepEquals, []string{chain3Path, chain4Path, hookPath})
+}
+
+func (s *snapExecSuite) TestSnapExecComponentWithoutHookError(c *C) {
+	dirs.SetRootDir(c.MkDir())
+
+	err := snapExec.ExecApp("snapname+comp", "42", "complete", []string{"foo"})
+	c.Assert(err, ErrorMatches, `snap-exec cannot run a snap component without a hook specified \(use --hook\)`)
+}
