@@ -28,6 +28,8 @@ import (
 	"mime/multipart"
 	"net/http"
 	"net/http/httptest"
+	"os"
+	"path/filepath"
 	"strconv"
 	"strings"
 	"time"
@@ -39,16 +41,20 @@ import (
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/client/clientutil"
 	"github.com/snapcore/snapd/daemon"
+	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/overlord/assertstate/assertstatetest"
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/devicestate"
 	"github.com/snapcore/snapd/overlord/devicestate/devicestatetest"
 	"github.com/snapcore/snapd/overlord/hookstate"
+	"github.com/snapcore/snapd/overlord/snapstate"
+	"github.com/snapcore/snapd/overlord/snapstate/snapstatetest"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/snaptest"
 )
 
-var modelDefaults = map[string]interface{}{
+var modelDefaults = map[string]any{
 	"architecture": "amd64",
 	"gadget":       "gadget",
 	"kernel":       "kernel",
@@ -70,7 +76,7 @@ func (s *modelSuite) TestPostRemodelUnhappy(c *check.C) {
 
 	req, err := http.NewRequest("POST", "/v2/model", bytes.NewBuffer(data))
 	c.Assert(err, check.IsNil)
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Assert(rspe.Status, check.Equals, 400)
 	c.Check(rspe.Message, check.Matches, "cannot decode new model assertion: .*")
 }
@@ -88,7 +94,7 @@ func (s *modelSuite) TestPostRemodelUnhappyWrongAssertion(c *check.C) {
 
 	req, err := http.NewRequest("POST", "/v2/model", bytes.NewBuffer(data))
 	c.Assert(err, check.IsNil)
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Assert(rspe.Status, check.Equals, 400)
 	c.Check(rspe.Message, check.Matches, "new model is not a model assertion: .*")
 }
@@ -107,7 +113,7 @@ func (s *modelSuite) testPostRemodel(c *check.C, offline bool) {
 	s.expectRootAccess()
 
 	oldModel := s.Brands.Model("my-brand", "my-old-model", modelDefaults)
-	newModel := s.Brands.Model("my-brand", "my-old-model", modelDefaults, map[string]interface{}{
+	newModel := s.Brands.Model("my-brand", "my-old-model", modelDefaults, map[string]any{
 		"revision": "2",
 	})
 
@@ -133,7 +139,7 @@ func (s *modelSuite) testPostRemodel(c *check.C, offline bool) {
 	defer restore()
 
 	var devicestateRemodelGotModel *asserts.Model
-	defer daemon.MockDevicestateRemodel(func(st *state.State, nm *asserts.Model, localSnaps []*snap.SideInfo, paths []string, opts devicestate.RemodelOptions) (*state.Change, error) {
+	defer daemon.MockDevicestateRemodel(func(st *state.State, nm *asserts.Model, opts devicestate.RemodelOptions) (*state.Change, error) {
 		c.Check(opts.Offline, check.Equals, offline)
 		devicestateRemodelGotModel = nm
 		chg := st.NewChange("remodel", "...")
@@ -153,7 +159,7 @@ func (s *modelSuite) testPostRemodel(c *check.C, offline bool) {
 	// devicestateRemodel
 	req, err := http.NewRequest("POST", "/v2/model", bytes.NewBuffer(data))
 	c.Assert(err, check.IsNil)
-	rsp := s.asyncReq(c, req, nil)
+	rsp := s.asyncReq(c, req, nil, actionIsExpected)
 	c.Assert(rsp.Status, check.Equals, 202)
 	c.Check(devicestateRemodelGotModel, check.DeepEquals, newModel)
 
@@ -193,7 +199,7 @@ func (s *modelSuite) TestPostRemodelWrongBody(c *check.C) {
 		req.Header.Set("Content-Type", "application/json")
 		c.Assert(err, check.IsNil)
 
-		rspe := s.errorReq(c, req, nil)
+		rspe := s.errorReq(c, req, nil, actionIsExpected)
 		c.Assert(rspe.Status, check.Equals, 400)
 		c.Assert(rspe.Kind, check.Equals, client.ErrorKind(""))
 		c.Assert(rspe.Value, check.IsNil)
@@ -215,7 +221,7 @@ func (s *modelSuite) TestPostRemodelWrongContentType(c *check.C) {
 	req.Header.Set("Content-Type", "footype")
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Assert(rspe.Status, check.Equals, 400)
 	c.Assert(rspe.Kind, check.Equals, client.ErrorKind(""))
 	c.Assert(rspe.Value, check.IsNil)
@@ -225,7 +231,7 @@ func (s *modelSuite) TestPostRemodelWrongContentType(c *check.C) {
 	req.Header.Set("Content-Type", "multipart/form-data")
 	c.Assert(err, check.IsNil)
 
-	rspe = s.errorReq(c, req, nil)
+	rspe = s.errorReq(c, req, nil, actionIsExpected)
 	c.Assert(rspe.Status, check.Equals, 400)
 	c.Assert(rspe.Kind, check.Equals, client.ErrorKind(""))
 	c.Assert(rspe.Value, check.IsNil)
@@ -243,7 +249,7 @@ func (s *modelSuite) TestGetModelNoModelAssertion(c *check.C) {
 
 	req, err := http.NewRequest("GET", "/v2/model", nil)
 	c.Assert(err, check.IsNil)
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Assert(rspe.Status, check.Equals, 404)
 	c.Assert(rspe.Kind, check.Equals, client.ErrorKindAssertionNotFound)
 	c.Assert(rspe.Value, check.Equals, "model")
@@ -272,7 +278,7 @@ func (s *modelSuite) TestGetModelHasModelAssertion(c *check.C) {
 	req, err := http.NewRequest("GET", "/v2/model", nil)
 	c.Assert(err, check.IsNil)
 	rec := httptest.NewRecorder()
-	s.req(c, req, nil).ServeHTTP(rec, req)
+	s.req(c, req, nil, actionIsExpected).ServeHTTP(rec, req)
 
 	// check that we get an assertion response
 	c.Check(rec.Code, check.Equals, 200, check.Commentf("body %q", rec.Body))
@@ -314,7 +320,7 @@ func (s *modelSuite) TestGetModelJSONHasModelAssertion(c *check.C) {
 	// make a new get request to the model endpoint with json as true
 	req, err := http.NewRequest("GET", "/v2/model?json=true", nil)
 	c.Assert(err, check.IsNil)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	// get the body and try to unmarshal into modelAssertJSON
 	c.Assert(rsp.Result, check.FitsTypeOf, clientutil.ModelAssertJSON{})
 
@@ -340,7 +346,7 @@ func (s *modelSuite) TestGetModelNoSerialAssertion(c *check.C) {
 
 	req, err := http.NewRequest("GET", "/v2/model/serial", nil)
 	c.Assert(err, check.IsNil)
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Assert(rspe.Status, check.Equals, 404)
 	c.Assert(rspe.Kind, check.Equals, client.ErrorKindAssertionNotFound)
 	c.Assert(rspe.Value, check.Equals, "serial")
@@ -370,7 +376,7 @@ func (s *modelSuite) TestGetModelHasSerialAssertion(c *check.C) {
 	assertstatetest.AddMany(st, s.Brands.AccountsAndKeys("my-brand")...)
 	s.mockModel(st, theModel)
 
-	serial, err := s.Brands.Signing("my-brand").Sign(asserts.SerialType, map[string]interface{}{
+	serial, err := s.Brands.Signing("my-brand").Sign(asserts.SerialType, map[string]any{
 		"authority-id":        "my-brand",
 		"brand-id":            "my-brand",
 		"model":               "my-old-model",
@@ -394,7 +400,7 @@ func (s *modelSuite) TestGetModelHasSerialAssertion(c *check.C) {
 	req, err := http.NewRequest("GET", "/v2/model/serial", nil)
 	c.Assert(err, check.IsNil)
 	rec := httptest.NewRecorder()
-	s.req(c, req, nil).ServeHTTP(rec, req)
+	s.req(c, req, nil, actionIsExpected).ServeHTTP(rec, req)
 
 	// check that we get an assertion response
 	c.Check(rec.Code, check.Equals, 200, check.Commentf("body %q", rec.Body))
@@ -438,7 +444,7 @@ func (s *modelSuite) TestGetModelJSONHasSerialAssertion(c *check.C) {
 	assertstatetest.AddMany(st, s.Brands.AccountsAndKeys("my-brand")...)
 	s.mockModel(st, theModel)
 
-	serial, err := s.Brands.Signing("my-brand").Sign(asserts.SerialType, map[string]interface{}{
+	serial, err := s.Brands.Signing("my-brand").Sign(asserts.SerialType, map[string]any{
 		"authority-id":        "my-brand",
 		"brand-id":            "my-brand",
 		"model":               "my-old-model",
@@ -461,7 +467,7 @@ func (s *modelSuite) TestGetModelJSONHasSerialAssertion(c *check.C) {
 	// make a new get request to the model endpoint with json as true
 	req, err := http.NewRequest("GET", "/v2/model/serial?json=true", nil)
 	c.Assert(err, check.IsNil)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	// get the body and try to unmarshal into modelAssertJSON
 	c.Assert(rsp.Result, check.FitsTypeOf, clientutil.ModelAssertJSON{})
 
@@ -482,7 +488,7 @@ func (s *userSuite) TestPostSerialBadAction(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/model/serial", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsUnexpected)
 	c.Check(rspe, check.DeepEquals, daemon.BadRequest(`unsupported serial action "what"`))
 }
 
@@ -499,7 +505,7 @@ func (s *userSuite) TestPostSerialForget(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/model/serial", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Result, check.IsNil)
 
 	c.Check(unregister, check.Equals, 1)
@@ -518,7 +524,7 @@ func (s *userSuite) TestPostSerialForgetNoRegistrationUntilReboot(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/model/serial", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Result, check.IsNil)
 
 	c.Check(unregister, check.Equals, 1)
@@ -533,7 +539,7 @@ func (s *userSuite) TestPostSerialForgetError(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/model/serial", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe, check.DeepEquals, daemon.InternalError(`forgetting serial failed: boom`))
 }
 
@@ -571,7 +577,7 @@ func (s *modelSuite) testPostOfflineRemodel(c *check.C, params *testPostOfflineR
 	s.expectRootAccess()
 
 	oldModel := s.Brands.Model("my-brand", "my-old-model", modelDefaults)
-	newModel := s.Brands.Model("my-brand", "my-old-model", modelDefaults, map[string]interface{}{
+	newModel := s.Brands.Model("my-brand", "my-old-model", modelDefaults, map[string]any{
 		"revision": "2",
 	})
 
@@ -599,13 +605,12 @@ func (s *modelSuite) testPostOfflineRemodel(c *check.C, params *testPostOfflineR
 	snapName := "snap1"
 	snapRev := 1001
 	var devicestateRemodelGotModel *asserts.Model
-	defer daemon.MockDevicestateRemodel(func(st *state.State, nm *asserts.Model,
-		localSnaps []*snap.SideInfo, paths []string, opts devicestate.RemodelOptions) (*state.Change, error) {
+	defer daemon.MockDevicestateRemodel(func(st *state.State, nm *asserts.Model, opts devicestate.RemodelOptions) (*state.Change, error) {
 		c.Check(opts.Offline, check.Equals, true)
-		c.Check(len(localSnaps), check.Equals, 1)
-		c.Check(localSnaps[0].RealName, check.Equals, snapName)
-		c.Check(localSnaps[0].Revision, check.Equals, snap.Revision{N: snapRev})
-		c.Check(strings.HasSuffix(paths[0],
+		c.Check(len(opts.LocalSnaps), check.Equals, 1)
+		c.Check(opts.LocalSnaps[0].SideInfo.RealName, check.Equals, snapName)
+		c.Check(opts.LocalSnaps[0].SideInfo.Revision, check.Equals, snap.Revision{N: snapRev})
+		c.Check(strings.HasSuffix(opts.LocalSnaps[0].Path,
 			"/var/lib/snapd/snaps/"+snapName+"_"+strconv.Itoa(snapRev)+".snap"),
 			check.Equals, true)
 
@@ -614,8 +619,12 @@ func (s *modelSuite) testPostOfflineRemodel(c *check.C, params *testPostOfflineR
 		return chg, nil
 	})()
 
-	sis := []*snap.SideInfo{{RealName: snapName, Revision: snap.Revision{N: snapRev}}}
-	defer daemon.MockSideloadSnapsInfo(sis)()
+	infos := []*snap.Info{{SideInfo: snap.SideInfo{
+		RealName: snapName,
+		Revision: snap.R(snapRev),
+	}}}
+
+	defer daemon.MockSideloadSnapsInfo(infos)()
 
 	// create a valid model assertion
 	c.Assert(err, check.IsNil)
@@ -627,7 +636,7 @@ func (s *modelSuite) testPostOfflineRemodel(c *check.C, params *testPostOfflineR
 	}
 
 	// valid revision assertion to make it part of the arguments
-	revAssert := assertstest.FakeAssertion(map[string]interface{}{
+	revAssert := assertstest.FakeAssertion(map[string]any{
 		"type":          "snap-revision",
 		"authority-id":  "can0nical",
 		"snap-id":       "snap-id-1",
@@ -648,11 +657,11 @@ func (s *modelSuite) testPostOfflineRemodel(c *check.C, params *testPostOfflineR
 	req.Header.Set("Content-Length", strconv.Itoa(body.Len()))
 
 	if params.badModel {
-		rsp := s.errorReq(c, req, nil)
+		rsp := s.errorReq(c, req, nil, actionIsExpected)
 		c.Assert(rsp.Status, check.Equals, 400)
 		c.Check(rsp.Error(), check.Equals, "cannot decode new model assertion: assertion content/signature separator not found (api)")
 	} else {
-		rsp := s.asyncReq(c, req, nil)
+		rsp := s.asyncReq(c, req, nil, actionIsExpected)
 		c.Assert(rsp.Status, check.Equals, 202)
 		c.Check(rsp.Change, check.DeepEquals, "1")
 		c.Check(devicestateRemodelGotModel, check.DeepEquals, newModel)
@@ -671,3 +680,148 @@ func (s *modelSuite) testPostOfflineRemodel(c *check.C, params *testPostOfflineR
 		c.Assert(soon, check.Equals, 1)
 	}
 }
+
+func (s *modelSuite) TestPostOfflineRemodelWithComponents(c *check.C) {
+	s.expectRootAccess()
+
+	oldModel := s.Brands.Model("my-brand", "my-old-model", modelDefaults)
+	newModel := s.Brands.Model("my-brand", "my-old-model", modelDefaults, map[string]any{
+		"revision": "2",
+	})
+
+	d := s.daemonWithOverlordMockAndStore()
+	hookMgr, err := hookstate.Manager(d.Overlord().State(), d.Overlord().TaskRunner())
+	c.Assert(err, check.IsNil)
+	deviceMgr, err := devicestate.Manager(d.Overlord().State(), hookMgr, d.Overlord().TaskRunner(), nil)
+	c.Assert(err, check.IsNil)
+	d.Overlord().AddManager(deviceMgr)
+
+	st := d.Overlord().State()
+	st.Lock()
+	defer st.Unlock()
+
+	st.Set("seeded", true)
+
+	assertstatetest.AddMany(st, s.StoreSigning.StoreAccountKey(""))
+	assertstatetest.AddMany(st, s.Brands.AccountsAndKeys("my-brand")...)
+	s.mockModel(st, oldModel)
+
+	signer := assertstest.NewStoreStack("can0nical", nil)
+	assertstatetest.AddMany(st, signer.StoreAccountKey(""))
+
+	account := assertstest.NewAccount(signer, "developer1", nil, "")
+	c.Assert(signer.Add(account), check.IsNil)
+
+	const snapName = "some-snap"
+	snapID := snaptest.AssertedSnapID(snapName)
+	snapFormData := make(map[string]string)
+
+	snapPath := snaptest.MakeTestSnapWithFiles(c, withComponents("name: some-snap\nversion: 1", []string{"comp"}), nil)
+	digest, size, err := asserts.SnapFileSHA3_384(snapPath)
+	c.Assert(err, check.IsNil)
+
+	content, err := os.ReadFile(snapPath)
+	c.Assert(err, check.IsNil)
+	snapFormData[filepath.Base(snapPath)] = string(content)
+
+	rev := mockStoreAssertion(c, signer, signer.AuthorityID, account.AccountID(), asserts.SnapRevisionType, map[string]any{
+		"snap-id":       snapID,
+		"snap-sha3-384": digest,
+		"developer-id":  account.AccountID(),
+		"snap-size":     strconv.Itoa(int(size)),
+		"snap-revision": "10",
+	})
+
+	decl := mockStoreAssertion(c, signer, signer.AuthorityID, account.AccountID(), asserts.SnapDeclarationType, map[string]any{
+		"series":       "16",
+		"snap-id":      snapID,
+		"snap-name":    snapName,
+		"publisher-id": account.AccountID(),
+	})
+
+	compPath, resRev, resPair := makeStandardComponent(c, signer, signer.AuthorityID, account.AccountID(), snapName, "comp")
+	content, err = os.ReadFile(compPath)
+	c.Assert(err, check.IsNil)
+	snapFormData[filepath.Base(compPath)] = string(content)
+
+	// we handle components that are not associated with any of the snaps that
+	// are being uploaded a little differently, this part of the test helps
+	// cover that case
+	extraSnapDecl := mockStoreAssertion(c, signer, signer.AuthorityID, account.AccountID(), asserts.SnapDeclarationType, map[string]any{
+		"series":       "16",
+		"snap-id":      snaptest.AssertedSnapID("other-snap"),
+		"snap-name":    "other-snap",
+		"publisher-id": account.AccountID(),
+	})
+	extraCompPath, extraResRev, extraResPair := makeStandardComponent(c, signer, signer.AuthorityID, account.AccountID(), "other-snap", "comp")
+	content, err = os.ReadFile(extraCompPath)
+	c.Assert(err, check.IsNil)
+	snapFormData[filepath.Base(extraCompPath)] = string(content)
+
+	snapstate.Set(st, "other-snap", &snapstate.SnapState{
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{{
+			RealName: "other-snap",
+			Revision: snap.R(10),
+			SnapID:   snaptest.AssertedSnapID("other-snap"),
+		}}),
+		Current: snap.R(10),
+		Active:  true,
+	})
+
+	var assertions strings.Builder
+	encoder := asserts.NewEncoder(&assertions)
+
+	for _, a := range []asserts.Assertion{rev, decl, resRev, resPair, extraSnapDecl, extraResRev, extraResPair, account} {
+		err := encoder.Encode(a)
+		c.Assert(err, check.IsNil)
+	}
+
+	fields := map[string][]string{
+		"new-model": {string(asserts.Encode(newModel))},
+		"assertion": {assertions.String()},
+	}
+	form, boundary := createFormData(c, fields, snapFormData)
+
+	defer daemon.MockDevicestateRemodel(func(st *state.State, nm *asserts.Model, opts devicestate.RemodelOptions) (*state.Change, error) {
+		c.Check(opts.Offline, check.Equals, true)
+		c.Assert(len(opts.LocalSnaps), check.Equals, 1)
+		c.Check(opts.LocalSnaps[0].SideInfo.RealName, check.Equals, snapName)
+		c.Check(opts.LocalSnaps[0].SideInfo.Revision, check.Equals, snap.Revision{N: 10})
+
+		snapPath := filepath.Join(dirs.SnapBlobDir, "some-snap_10.snap")
+		c.Check(strings.HasSuffix(opts.LocalSnaps[0].Path, snapPath), check.Equals, true)
+
+		c.Assert(len(opts.LocalComponents), check.Equals, 2)
+		compPath := filepath.Join(dirs.SnapBlobDir, "some-snap+comp_20.comp")
+		c.Check(strings.HasSuffix(opts.LocalComponents[0].Path, compPath), check.Equals, true)
+
+		extraCompPath := filepath.Join(dirs.SnapBlobDir, "other-snap+comp_20.comp")
+		c.Check(strings.HasSuffix(opts.LocalComponents[1].Path, extraCompPath), check.Equals, true)
+
+		c.Check(nm, check.DeepEquals, newModel)
+
+		chg := st.NewChange("remodel", "...")
+		return chg, nil
+	})()
+
+	req, err := http.NewRequest("POST", "/v2/model", &form)
+	c.Assert(err, check.IsNil)
+	req.Header.Set("Content-Type", "multipart/form-data; boundary="+boundary)
+	req.Header.Set("Content-Length", strconv.Itoa(form.Len()))
+
+	st.Unlock()
+	rsp := s.asyncReq(c, req, nil, actionIsExpected)
+	st.Lock()
+
+	c.Assert(rsp.Status, check.Equals, 202)
+	c.Check(rsp.Change, check.DeepEquals, "1")
+
+	chg := st.Change(rsp.Change)
+	c.Assert(chg, check.NotNil)
+
+	c.Assert(st.Changes(), check.HasLen, 1)
+	chg1 := st.Changes()[0]
+	c.Assert(chg, check.DeepEquals, chg1)
+	c.Assert(chg.Kind(), check.Equals, "remodel")
+	c.Assert(chg.Err(), check.IsNil)
+}
