@@ -46,7 +46,7 @@ type noticesSuite struct {
 func (s *noticesSuite) SetUpTest(c *C) {
 	s.apiBaseSuite.SetUpTest(c)
 
-	s.expectReadAccess(daemon.InterfaceOpenAccess{Interfaces: []string{"snap-refresh-observe"}})
+	s.expectReadAccess(daemon.InterfaceOpenAccess{Interfaces: []string{"snap-refresh-observe", "snap-interfaces-requests-control"}})
 	s.expectWriteAccess(daemon.OpenAccess{})
 }
 
@@ -106,7 +106,7 @@ func (s *noticesSuite) testNoticesFilter(c *C, makeQuery func(after time.Time) u
 	req, err := http.NewRequest("GET", "/v2/notices?"+query.Encode(), nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=0;socket=%s;", dirs.SnapdSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notices, ok := rsp.Result.([]*state.Notice)
@@ -153,7 +153,7 @@ func (s *noticesSuite) TestNoticesFilterMultipleTypes(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices?types=change-update&types=warning,warning&types=refresh-inhibit", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;", dirs.SnapdSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notices, ok := rsp.Result.([]*state.Notice)
@@ -182,7 +182,7 @@ func (s *noticesSuite) TestNoticesFilterMultipleKeys(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices?keys=456&keys=danger", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;", dirs.SnapdSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notices, ok := rsp.Result.([]*state.Notice)
@@ -209,7 +209,7 @@ func (s *noticesSuite) TestNoticesFilterInvalidTypes(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices?types=foo&types=warning&types=bar,baz", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;", dirs.SnapdSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notices, ok := rsp.Result.([]*state.Notice)
@@ -223,7 +223,7 @@ func (s *noticesSuite) TestNoticesFilterInvalidTypes(c *C) {
 	req, err = http.NewRequest("GET", "/v2/notices?types=foo&types=bar,baz", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=1000;socket=;"
-	rsp = s.syncReq(c, req, nil)
+	rsp = s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notices, ok = rsp.Result.([]*state.Notice)
@@ -236,10 +236,13 @@ func (s *noticesSuite) TestNoticesShowsTypesAllowedForSnap(c *C) {
 
 	st := s.d.Overlord().State()
 	st.Lock()
+	addNotice(c, st, nil, state.InterfacesRequestsPromptNotice, "abc", nil)
+	addNotice(c, st, nil, state.InterfacesRequestsRuleUpdateNotice, "xyz", nil)
 	addNotice(c, st, nil, state.ChangeUpdateNotice, "123", nil)
 	addNotice(c, st, nil, state.RefreshInhibitNotice, "-", nil)
 	addNotice(c, st, nil, state.WarningNotice, "danger", nil)
 	addNotice(c, st, nil, state.SnapRunInhibitNotice, "snap-name", nil)
+	addNotice(c, st, nil, state.InterfacesRequestsPromptNotice, "def", nil)
 	st.Unlock()
 
 	// Check that a snap request without specifying types filter only shows
@@ -249,7 +252,7 @@ func (s *noticesSuite) TestNoticesShowsTypesAllowedForSnap(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=;", dirs.SnapSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 	notices, ok := rsp.Result.([]*state.Notice)
 	c.Assert(ok, Equals, true)
@@ -259,7 +262,7 @@ func (s *noticesSuite) TestNoticesShowsTypesAllowedForSnap(c *C) {
 	req, err = http.NewRequest("GET", "/v2/notices", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-refresh-observe;", dirs.SnapSocket)
-	rsp = s.syncReq(c, req, nil)
+	rsp = s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 	notices, ok = rsp.Result.([]*state.Notice)
 	c.Assert(ok, Equals, true)
@@ -274,6 +277,29 @@ func (s *noticesSuite) TestNoticesShowsTypesAllowedForSnap(c *C) {
 	c.Check(seenNoticeType["change-update"], Equals, 1)
 	c.Check(seenNoticeType["refresh-inhibit"], Equals, 1)
 	c.Check(seenNoticeType["snap-run-inhibit"], Equals, 1)
+
+	// Check that multiple interfaces allow accessing notice types granted by
+	// any of the connected interfaces
+	req, err = http.NewRequest("GET", "/v2/notices", nil)
+	c.Assert(err, IsNil)
+	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-refresh-observe&snap-interfaces-requests-control;", dirs.SnapSocket)
+	rsp = s.syncReq(c, req, nil, actionIsExpected)
+	c.Check(rsp.Status, Equals, 200)
+	notices, ok = rsp.Result.([]*state.Notice)
+	c.Assert(ok, Equals, true)
+	c.Assert(notices, HasLen, 6)
+
+	seenNoticeType = make(map[string]int)
+	for _, notice := range notices {
+		n := noticeToMap(c, notice)
+		noticeType := n["type"].(string)
+		seenNoticeType[noticeType]++
+	}
+	c.Check(seenNoticeType["change-update"], Equals, 1)
+	c.Check(seenNoticeType["refresh-inhibit"], Equals, 1)
+	c.Check(seenNoticeType["snap-run-inhibit"], Equals, 1)
+	c.Check(seenNoticeType["interfaces-requests-prompt"], Equals, 2)
+	c.Check(seenNoticeType["interfaces-requests-rule-update"], Equals, 1)
 }
 
 func (s *noticesSuite) TestNoticesFilterTypesForSnap(c *C) {
@@ -294,7 +320,7 @@ func (s *noticesSuite) TestNoticesFilterTypesForSnap(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices?types=change-update,refresh-inhibit,snap-run-inhibit", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-refresh-observe;", dirs.SnapSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 	notices, ok := rsp.Result.([]*state.Notice)
 	c.Assert(ok, Equals, true)
@@ -329,42 +355,44 @@ func (s *noticesSuite) TestNoticesFilterTypesForSnapForbidden(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices?types=change-update,warning", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-refresh-observe;", dirs.SnapSocket)
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 
 	// snap-refresh-observe doesn't give access to warning notices.
 	req, err = http.NewRequest("GET", "/v2/notices?types=warning", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-refresh-observe;", dirs.SnapSocket)
-	rsp = s.errorReq(c, req, nil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 
-	// snap-themes-control doesn't give access to change-update notices.
-	req, err = http.NewRequest("GET", "/v2/notices?types=change-update", nil)
-	c.Assert(err, IsNil)
-	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-themes-control;", dirs.SnapSocket)
-	rsp = s.errorReq(c, req, nil)
-	c.Check(rsp.Status, Equals, 403)
+	for _, iface := range []string{"snap-themes-control", "snap-interfaces-requests-control"} {
+		// neither interface gives access to change-update notices.
+		req, err = http.NewRequest("GET", "/v2/notices?types=change-update", nil)
+		c.Assert(err, IsNil)
+		req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=%s;", dirs.SnapSocket, iface)
+		rsp = s.errorReq(c, req, nil, actionIsExpected)
+		c.Check(rsp.Status, Equals, 403)
 
-	// snap-themes-control doesn't give access to refresh-inhibit notices.
-	req, err = http.NewRequest("GET", "/v2/notices?types=refresh-inhibit", nil)
-	c.Assert(err, IsNil)
-	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-themes-control;", dirs.SnapSocket)
-	rsp = s.errorReq(c, req, nil)
-	c.Check(rsp.Status, Equals, 403)
+		// neither interface gives access to refresh-inhibit notices.
+		req, err = http.NewRequest("GET", "/v2/notices?types=refresh-inhibit", nil)
+		c.Assert(err, IsNil)
+		req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=%s;", dirs.SnapSocket, iface)
+		rsp = s.errorReq(c, req, nil, actionIsExpected)
+		c.Check(rsp.Status, Equals, 403)
 
-	// snap-themes-control doesn't give access to snap-run-inhibit notices.
-	req, err = http.NewRequest("GET", "/v2/notices?types=snap-run-inhibit", nil)
-	c.Assert(err, IsNil)
-	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-themes-control;", dirs.SnapSocket)
-	rsp = s.errorReq(c, req, nil)
-	c.Check(rsp.Status, Equals, 403)
+		// neither interface access to snap-run-inhibit notices.
+		req, err = http.NewRequest("GET", "/v2/notices?types=snap-run-inhibit", nil)
+		c.Assert(err, IsNil)
+		req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=%s;", dirs.SnapSocket, iface)
+		rsp = s.errorReq(c, req, nil, actionIsExpected)
+		c.Check(rsp.Status, Equals, 403)
+	}
 
 	// No interfaces connected.
 	req, err = http.NewRequest("GET", "/v2/notices?types=change-update,refresh-inhibit,snap-run-inhibit", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=;", dirs.SnapSocket)
-	rsp = s.errorReq(c, req, nil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 }
 
@@ -389,7 +417,7 @@ func (s *noticesSuite) TestNoticesUserIDAdminDefault(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=0;socket=%s;", dirs.SnapdSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notices, ok := rsp.Result.([]*state.Notice)
@@ -428,7 +456,7 @@ func (s *noticesSuite) TestNoticesUserIDAdminFilter(c *C) {
 		req, err := http.NewRequest("GET", reqUrl, nil)
 		c.Assert(err, IsNil)
 		req.RemoteAddr = fmt.Sprintf("pid=100;uid=0;socket=%s;", dirs.SnapdSocket)
-		rsp := s.syncReq(c, req, nil)
+		rsp := s.syncReq(c, req, nil, actionIsExpected)
 		c.Check(rsp.Status, Equals, 200)
 
 		notices, ok := rsp.Result.([]*state.Notice)
@@ -462,7 +490,7 @@ func (s *noticesSuite) TestNoticesUserIDNonAdminDefault(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;", dirs.SnapdSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notices, ok := rsp.Result.([]*state.Notice)
@@ -490,7 +518,7 @@ func (s *noticesSuite) TestNoticesUserIDNonAdminFilter(c *C) {
 	req, err := http.NewRequest("GET", reqUrl, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=1000;socket=;"
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 }
 
@@ -516,7 +544,7 @@ func (s *noticesSuite) TestNoticesUsersAdminFilter(c *C) {
 	req, err := http.NewRequest("GET", reqUrl, nil)
 	c.Check(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=0;socket=%s;", dirs.SnapdSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notices, ok := rsp.Result.([]*state.Notice)
@@ -550,7 +578,7 @@ func (s *noticesSuite) TestNoticesUsersNonAdminFilter(c *C) {
 	req, err := http.NewRequest("GET", reqUrl, nil)
 	c.Check(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=1000;socket=;"
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 }
 
@@ -566,7 +594,7 @@ func (s *noticesSuite) TestNoticesUnknownRequestUID(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=;socket=;"
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 }
 
@@ -585,7 +613,7 @@ func (s *noticesSuite) TestNoticesWait(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices?timeout="+timeout, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;", dirs.SnapdSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notices, ok := rsp.Result.([]*state.Notice)
@@ -603,7 +631,7 @@ func (s *noticesSuite) TestNoticesTimeout(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices?timeout=1ms", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=1000;socket=;"
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notices, ok := rsp.Result.([]*state.Notice)
@@ -630,8 +658,8 @@ func (s *noticesSuite) TestNoticesRequestCancelled(c *C) {
 	req, err := http.NewRequestWithContext(ctx, "GET", "/v2/notices?timeout="+reqTimeout.String(), nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;", dirs.SnapdSocket)
-	rsp := s.errorReq(c, req, nil)
-	c.Check(rsp.Status, Equals, 400)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
+	c.Check(rsp.Status, Equals, 500)
 	c.Check(rsp.Message, Matches, "request canceled")
 
 	elapsed := time.Since(start)
@@ -677,7 +705,7 @@ func (s *noticesSuite) testNoticesBadRequest(c *C, query, errorMatch string) {
 	req, err := http.NewRequest("GET", "/v2/notices?"+query, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=0;socket=%s;", dirs.SnapdSocket)
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 400)
 	c.Assert(rsp.Message, Matches, errorMatch)
 }
@@ -835,7 +863,7 @@ func (s *noticesSuite) TestAddNotice(c *C) {
 	req, err := http.NewRequest("POST", "/v2/notices", bytes.NewReader(body))
 	c.Assert(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=1000;socket=;"
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Assert(rsp.Status, Equals, 200)
 
 	resultBytes, err := json.Marshal(rsp.Result)
@@ -884,7 +912,7 @@ func (s *noticesSuite) TestAddNoticeInvalidRequestUid(c *C) {
 	req, err := http.NewRequest("POST", "/v2/notices", bytes.NewReader(body))
 	c.Assert(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=;socket=;"
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
 	c.Assert(rsp.Status, Equals, 403)
 }
 
@@ -931,7 +959,7 @@ func (s *noticesSuite) testAddNoticeBadRequest(c *C, body, errorMatch string) {
 	req, err := http.NewRequest("POST", "/v2/notices", strings.NewReader(body))
 	c.Assert(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=1000;socket=;"
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionExpectedBool(!strings.Contains(errorMatch, "invalid action")))
 	c.Check(rsp.Status, Equals, 400)
 	c.Assert(rsp.Message, Matches, errorMatch)
 }
@@ -981,11 +1009,11 @@ func (s *noticesSuite) testAddNoticesSnapCmd(c *C, exePath string, shouldFail bo
 	req.RemoteAddr = "pid=100;uid=1000;socket=;"
 
 	if shouldFail {
-		rsp := s.errorReq(c, req, nil)
+		rsp := s.errorReq(c, req, nil, actionIsExpected)
 		c.Check(rsp.Status, Equals, 403)
 		c.Assert(rsp.Message, Matches, "only snap command can record notices")
 	} else {
-		rsp := s.syncReq(c, req, nil)
+		rsp := s.syncReq(c, req, nil, actionIsExpected)
 		c.Assert(rsp.Status, Equals, 200)
 	}
 }
@@ -1007,7 +1035,7 @@ func (s *noticesSuite) TestNotice(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices/"+noticeIDPublic, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;", dirs.SnapdSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notice, ok := rsp.Result.(*state.Notice)
@@ -1020,7 +1048,7 @@ func (s *noticesSuite) TestNotice(c *C) {
 	req, err = http.NewRequest("GET", "/v2/notices/"+noticeIDPrivate, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;", dirs.SnapdSocket)
-	rsp = s.syncReq(c, req, nil)
+	rsp = s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 200)
 
 	notice, ok = rsp.Result.(*state.Notice)
@@ -1037,7 +1065,7 @@ func (s *noticesSuite) TestNoticeNotFound(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices/1234", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=1000;socket=;"
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 404)
 }
 
@@ -1047,7 +1075,7 @@ func (s *noticesSuite) TestNoticeUnknownRequestUID(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices/1234", nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=;socket=;"
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 }
 
@@ -1064,7 +1092,7 @@ func (s *noticesSuite) TestNoticeAdminAllowed(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices/"+noticeID, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=0;socket=%s;", dirs.SnapdSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Assert(rsp.Status, Equals, 200)
 
 	notice, ok := rsp.Result.(*state.Notice)
@@ -1088,7 +1116,7 @@ func (s *noticesSuite) TestNoticeNonAdminNotAllowed(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices/"+noticeID, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = "pid=100;uid=1001;socket=;"
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 }
 
@@ -1107,7 +1135,7 @@ func (s *noticesSuite) TestNoticeSnapAllowed(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices/"+changeUpdateNoticeID, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1001;socket=%s;iface=snap-refresh-observe;", dirs.SnapSocket)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Assert(rsp.Status, Equals, 200)
 
 	notice, ok := rsp.Result.(*state.Notice)
@@ -1120,7 +1148,7 @@ func (s *noticesSuite) TestNoticeSnapAllowed(c *C) {
 	req, err = http.NewRequest("GET", "/v2/notices/"+refreshInhibitNoticeID, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1001;socket=%s;iface=snap-refresh-observe;", dirs.SnapSocket)
-	rsp = s.syncReq(c, req, nil)
+	rsp = s.syncReq(c, req, nil, actionIsExpected)
 	c.Assert(rsp.Status, Equals, 200)
 
 	notice, ok = rsp.Result.(*state.Notice)
@@ -1147,33 +1175,33 @@ func (s *noticesSuite) TestNoticeSnapNotAllowed(c *C) {
 	req, err := http.NewRequest("GET", "/v2/notices/"+warningNoticeID, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-refresh-observe;", dirs.SnapSocket)
-	rsp := s.errorReq(c, req, nil)
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 
 	// snap-themes-control doesn't give access to change-update notices.
 	req, err = http.NewRequest("GET", "/v2/notices/"+changeUpdateNoticeID, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-themes-control;", dirs.SnapSocket)
-	rsp = s.errorReq(c, req, nil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 
 	// snap-themes-control doesn't give access to refresh-inhibit notices.
 	req, err = http.NewRequest("GET", "/v2/notices/"+refreshInhibitNoticeID, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=snap-themes-control;", dirs.SnapSocket)
-	rsp = s.errorReq(c, req, nil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 
 	// No interface connected.
 	req, err = http.NewRequest("GET", "/v2/notices/"+changeUpdateNoticeID, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=;", dirs.SnapSocket)
-	rsp = s.errorReq(c, req, nil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 	req, err = http.NewRequest("GET", "/v2/notices/"+refreshInhibitNoticeID, nil)
 	c.Assert(err, IsNil)
 	req.RemoteAddr = fmt.Sprintf("pid=100;uid=1000;socket=%s;iface=;", dirs.SnapSocket)
-	rsp = s.errorReq(c, req, nil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, Equals, 403)
 }
 
