@@ -23,7 +23,6 @@ import (
 	"bytes"
 	"fmt"
 	"net/http"
-	"os/user"
 	"time"
 
 	"gopkg.in/check.v1"
@@ -32,6 +31,7 @@ import (
 	"github.com/snapcore/snapd/asserts/assertstest"
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/daemon"
+	"github.com/snapcore/snapd/osutil/user"
 	"github.com/snapcore/snapd/overlord/assertstate/assertstatetest"
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/configstate/config"
@@ -119,7 +119,7 @@ func (s *userSuite) TestLoginUser(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 
 	state.Lock()
 	user, err := auth.User(state, 1)
@@ -162,7 +162,7 @@ func (s *userSuite) TestLoginUserWithUsername(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 
 	state.Lock()
 	user, err := auth.User(state, 1)
@@ -215,7 +215,7 @@ func (s *userSuite) TestLoginUserNoEmailWithExistentLocalUser(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, localUser)
+	rsp := s.syncReq(c, req, localUser, actionIsExpected)
 
 	expected := daemon.UserResponseData{
 		ID:       1,
@@ -263,7 +263,7 @@ func (s *userSuite) TestLoginUserWithExistentLocalUser(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, localUser)
+	rsp := s.syncReq(c, req, localUser, actionIsExpected)
 
 	expected := daemon.UserResponseData{
 		ID:       1,
@@ -312,7 +312,7 @@ func (s *userSuite) TestLoginUserNewEmailWithExistentLocalUser(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, localUser)
+	rsp := s.syncReq(c, req, localUser, actionIsExpected)
 
 	expected := daemon.UserResponseData{
 		ID:       1,
@@ -356,7 +356,7 @@ func (s *userSuite) TestLogoutUser(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/logout", nil)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, user)
+	rsp := s.syncReq(c, req, user, actionIsExpected)
 	c.Check(rsp.Status, check.Equals, 200)
 
 	state.Lock()
@@ -372,7 +372,7 @@ func (s *userSuite) TestLoginUserBadRequest(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Status, check.Equals, 400)
 	c.Check(rspe.Message, check.Not(check.Equals), "")
 }
@@ -384,7 +384,7 @@ func (s *userSuite) TestLoginUserNotEmailish(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Status, check.Equals, 400)
 	c.Check(rspe.Message, testutil.Contains, "please use a valid email address")
 }
@@ -397,7 +397,7 @@ func (s *userSuite) TestLoginUserDeveloperAPIError(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Status, check.Equals, 401)
 	c.Check(rspe.Message, testutil.Contains, "error-from-login-user")
 }
@@ -410,7 +410,7 @@ func (s *userSuite) TestLoginUserTwoFactorRequiredError(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Status, check.Equals, 401)
 	c.Check(rspe.Kind, check.Equals, client.ErrorKindTwoFactorRequired)
 }
@@ -423,7 +423,7 @@ func (s *userSuite) TestLoginUserTwoFactorFailedError(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Status, check.Equals, 401)
 	c.Check(rspe.Kind, check.Equals, client.ErrorKindTwoFactorFailed)
 }
@@ -436,7 +436,7 @@ func (s *userSuite) TestLoginUserInvalidCredentialsError(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Status, check.Equals, 401)
 	c.Check(rspe.Message, check.Equals, "invalid credentials")
 }
@@ -449,7 +449,7 @@ func (s *userSuite) TestLoginUserInvalidAuthDataError(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Status, check.Equals, 400)
 	c.Check(rspe.Kind, check.Equals, client.ErrorKindInvalidAuthData)
 	c.Check(rspe.Value, check.DeepEquals, s.err)
@@ -463,7 +463,7 @@ func (s *userSuite) TestLoginUserPasswordPolicyError(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/login", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Status, check.Equals, 401)
 	c.Check(rspe.Kind, check.Equals, client.ErrorKindPasswordPolicy)
 	c.Check(rspe.Value, check.DeepEquals, s.err)
@@ -498,7 +498,7 @@ func (s *userSuite) testCreateUser(c *check.C, oldWay bool) {
 	})()
 
 	var req *http.Request
-	var expected interface{}
+	var expected any
 	expectedItem := daemon.UserResponseData{
 		Username: expectedUsername,
 		SSHKeys: []string{
@@ -521,7 +521,7 @@ func (s *userSuite) testCreateUser(c *check.C, oldWay bool) {
 		expected = []daemon.UserResponseData{expectedItem}
 	}
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Result, check.FitsTypeOf, expected)
 	c.Check(rsp.Result, check.DeepEquals, expected)
 	c.Check(deviceStateCreateUserCalled, check.Equals, true)
@@ -550,7 +550,7 @@ func (s *userSuite) testCreateUserErr(c *check.C, internalErr bool) {
 	req, err := http.NewRequest("POST", "/v2/create-user", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(called, check.Equals, 1)
 	if internalErr {
 		c.Check(rspe.Status, check.Equals, 500)
@@ -572,7 +572,7 @@ func (s *userSuite) testNoUserAdmin(c *check.C, endpoint string) {
 	req, err := http.NewRequest("POST", endpoint, buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 
 	const noUserAdmin = "system user administration via snapd is not allowed on this system"
 	switch endpoint {
@@ -590,7 +590,7 @@ func (s *userSuite) TestPostUserBadBody(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/users", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Message, check.Matches, "cannot decode user action data from request body: .*")
 }
 
@@ -599,7 +599,7 @@ func (s *userSuite) TestPostUserBadAfterBody(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/users", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe, check.DeepEquals, daemon.BadRequest("spurious content after user action"))
 }
 
@@ -608,7 +608,7 @@ func (s *userSuite) TestPostUserNoAction(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/users", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe, check.DeepEquals, daemon.BadRequest("missing user action"))
 }
 
@@ -617,7 +617,7 @@ func (s *userSuite) TestPostUserBadAction(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/users", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsUnexpected)
 	c.Check(rspe, check.DeepEquals, daemon.BadRequest(`unsupported user action "patatas"`))
 }
 
@@ -655,7 +655,7 @@ func (s *userSuite) testpostUserActionRemoveDelUserErr(c *check.C, internalErr b
 	req, err := http.NewRequest("POST", "/v2/users", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(called, check.Equals, 1)
 	if internalErr {
 		c.Check(rspe.Status, check.Equals, 500)
@@ -682,12 +682,12 @@ func (s *userSuite) TestPostUserActionRemove(c *check.C) {
 	buf := bytes.NewBufferString(`{"action":"remove","username":"some-user"}`)
 	req, err := http.NewRequest("POST", "/v2/users", buf)
 	c.Assert(err, check.IsNil)
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Status, check.Equals, 200)
 	expected := []daemon.UserResponseData{
 		{ID: expectedID, Username: expectedUsername, Email: expectedEmail},
 	}
-	c.Check(rsp.Result, check.DeepEquals, map[string]interface{}{
+	c.Check(rsp.Result, check.DeepEquals, map[string]any{
 		"removed": expected,
 	})
 	c.Check(called, check.Equals, 1)
@@ -696,7 +696,7 @@ func (s *userSuite) TestPostUserActionRemove(c *check.C) {
 func (s *userSuite) setupSigner(accountID string, signerPrivKey asserts.PrivateKey) *assertstest.SigningDB {
 	st := s.d.Overlord().State()
 
-	signerSigning := s.Brands.Register(accountID, signerPrivKey, map[string]interface{}{
+	signerSigning := s.Brands.Register(accountID, signerPrivKey, map[string]any{
 		"account-id":   accountID,
 		"verification": "verified",
 	})
@@ -713,7 +713,7 @@ var (
 	unknownPrivKey, _ = assertstest.GenerateKey(752)
 )
 
-func (s *userSuite) makeSystemUsers(c *check.C, systemUsers []map[string]interface{}) {
+func (s *userSuite) makeSystemUsers(c *check.C, systemUsers []map[string]any) {
 	st := s.d.Overlord().State()
 	st.Lock()
 	defer st.Unlock()
@@ -724,12 +724,12 @@ func (s *userSuite) makeSystemUsers(c *check.C, systemUsers []map[string]interfa
 	s.setupSigner("partner", partnerPrivKey)
 	s.setupSigner("unknown", unknownPrivKey)
 
-	model := s.Brands.Model("my-brand", "my-model", map[string]interface{}{
+	model := s.Brands.Model("my-brand", "my-model", map[string]any{
 		"architecture":          "amd64",
 		"gadget":                "pc",
 		"kernel":                "pc-kernel",
-		"required-snaps":        []interface{}{"required-snap1"},
-		"system-user-authority": []interface{}{"my-brand", "partner"},
+		"required-snaps":        []any{"required-snap1"},
+		"system-user-authority": []any{"my-brand", "partner"},
 	})
 	// now add model related stuff to the system
 	assertstatetest.AddMany(st, model)
@@ -737,7 +737,7 @@ func (s *userSuite) makeSystemUsers(c *check.C, systemUsers []map[string]interfa
 	deviceKey, _ := assertstest.GenerateKey(752)
 	encDevKey, err := asserts.EncodePublicKey(deviceKey.PublicKey())
 	c.Assert(err, check.IsNil)
-	serial, err := s.Brands.Signing("my-brand").Sign(asserts.SerialType, map[string]interface{}{
+	serial, err := s.Brands.Signing("my-brand").Sign(asserts.SerialType, map[string]any{
 		"authority-id":        "my-brand",
 		"brand-id":            "my-brand",
 		"model":               "my-model",
@@ -765,12 +765,12 @@ func (s *userSuite) makeSystemUsers(c *check.C, systemUsers []map[string]interfa
 	c.Assert(err, check.IsNil)
 }
 
-var goodUser = map[string]interface{}{
+var goodUser = map[string]any{
 	"authority-id": "my-brand",
 	"brand-id":     "my-brand",
 	"email":        "foo@bar.com",
-	"series":       []interface{}{"16", "18"},
-	"models":       []interface{}{"my-model", "other-model"},
+	"series":       []any{"16", "18"},
+	"models":       []any{"my-model", "other-model"},
 	"name":         "Boring Guy",
 	"username":     "guy",
 	"password":     "$6$salt$hash",
@@ -778,12 +778,12 @@ var goodUser = map[string]interface{}{
 	"until":        time.Now().Add(24 * 30 * time.Hour).Format(time.RFC3339),
 }
 
-var partnerUser = map[string]interface{}{
+var partnerUser = map[string]any{
 	"authority-id": "partner",
 	"brand-id":     "my-brand",
 	"email":        "p@partner.com",
-	"series":       []interface{}{"16", "18"},
-	"models":       []interface{}{"my-model"},
+	"series":       []any{"16", "18"},
+	"models":       []any{"my-model"},
 	"name":         "Partner Guy",
 	"username":     "partnerguy",
 	"password":     "$6$salt$hash",
@@ -791,14 +791,14 @@ var partnerUser = map[string]interface{}{
 	"until":        time.Now().Add(24 * 30 * time.Hour).Format(time.RFC3339),
 }
 
-var serialUser = map[string]interface{}{
+var serialUser = map[string]any{
 	"format":       "1",
 	"authority-id": "my-brand",
 	"brand-id":     "my-brand",
 	"email":        "serial@bar.com",
-	"series":       []interface{}{"16", "18"},
-	"models":       []interface{}{"my-model"},
-	"serials":      []interface{}{"serialserial"},
+	"series":       []any{"16", "18"},
+	"models":       []any{"my-model"},
+	"serials":      []any{"serialserial"},
 	"name":         "Serial Guy",
 	"username":     "goodserialguy",
 	"password":     "$6$salt$hash",
@@ -806,13 +806,13 @@ var serialUser = map[string]interface{}{
 	"until":        time.Now().Add(24 * 30 * time.Hour).Format(time.RFC3339),
 }
 
-var badUser = map[string]interface{}{
+var badUser = map[string]any{
 	// bad user (not valid for this model)
 	"authority-id": "my-brand",
 	"brand-id":     "my-brand",
 	"email":        "foobar@bar.com",
-	"series":       []interface{}{"16", "18"},
-	"models":       []interface{}{"non-of-the-models-i-have"},
+	"series":       []any{"16", "18"},
+	"models":       []any{"non-of-the-models-i-have"},
 	"name":         "Random Gal",
 	"username":     "gal",
 	"password":     "$6$salt$hash",
@@ -820,14 +820,14 @@ var badUser = map[string]interface{}{
 	"until":        time.Now().Add(24 * 30 * time.Hour).Format(time.RFC3339),
 }
 
-var badUserNoMatchingSerial = map[string]interface{}{
+var badUserNoMatchingSerial = map[string]any{
 	"format":       "1",
 	"authority-id": "my-brand",
 	"brand-id":     "my-brand",
 	"email":        "noserial@bar.com",
-	"series":       []interface{}{"16", "18"},
-	"models":       []interface{}{"my-model"},
-	"serials":      []interface{}{"different-serialserial"},
+	"series":       []any{"16", "18"},
+	"models":       []any{"my-model"},
+	"serials":      []any{"different-serialserial"},
 	"name":         "No Serial Guy",
 	"username":     "noserial",
 	"password":     "$6$salt$hash",
@@ -835,12 +835,12 @@ var badUserNoMatchingSerial = map[string]interface{}{
 	"until":        time.Now().Add(24 * 30 * time.Hour).Format(time.RFC3339),
 }
 
-var unknownUser = map[string]interface{}{
+var unknownUser = map[string]any{
 	"authority-id": "unknown",
 	"brand-id":     "my-brand",
 	"email":        "x@partner.com",
-	"series":       []interface{}{"16", "18"},
-	"models":       []interface{}{"my-model"},
+	"series":       []any{"16", "18"},
+	"models":       []any{"my-model"},
 	"name":         "XGuy",
 	"username":     "xguy",
 	"password":     "$6$salt$hash",
@@ -852,19 +852,19 @@ func (s *userSuite) TestPostCreateUserFromAssertionAllKnownClassicErrors(c *chec
 	restore := release.MockOnClassic(true)
 	defer restore()
 
-	s.makeSystemUsers(c, []map[string]interface{}{goodUser})
+	s.makeSystemUsers(c, []map[string]any{goodUser})
 
 	// do it!
 	buf := bytes.NewBufferString(`{"known":true}`)
 	req, err := http.NewRequest("POST", "/v2/create-user", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Message, check.Matches, `cannot create user: device is a classic system`)
 }
 
 func (s *userSuite) TestPostCreateUserFromAssertionAllKnownButOwnedErrors(c *check.C) {
-	s.makeSystemUsers(c, []map[string]interface{}{goodUser})
+	s.makeSystemUsers(c, []map[string]any{goodUser})
 
 	st := s.d.Overlord().State()
 	st.Lock()
@@ -882,12 +882,12 @@ func (s *userSuite) TestPostCreateUserFromAssertionAllKnownButOwnedErrors(c *che
 	req, err := http.NewRequest("POST", "/v2/create-user", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Message, check.Matches, `cannot create user: device already managed`)
 }
 
 func (s *userSuite) TestPostCreateUserAutomaticManagedDoesNotActOrError(c *check.C) {
-	s.makeSystemUsers(c, []map[string]interface{}{goodUser})
+	s.makeSystemUsers(c, []map[string]any{goodUser})
 
 	st := s.d.Overlord().State()
 	st.Lock()
@@ -905,7 +905,7 @@ func (s *userSuite) TestPostCreateUserAutomaticManagedDoesNotActOrError(c *check
 	req, err := http.NewRequest("POST", "/v2/create-user", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 
 	// expecting an empty reply
 	expected := []daemon.UserResponseData{}
@@ -914,7 +914,7 @@ func (s *userSuite) TestPostCreateUserAutomaticManagedDoesNotActOrError(c *check
 }
 
 func (s *userSuite) TestPostCreateUserAutomaticDisabled(c *check.C) {
-	s.makeSystemUsers(c, []map[string]interface{}{goodUser})
+	s.makeSystemUsers(c, []map[string]any{goodUser})
 
 	// disable automatic user creation
 	st := s.d.Overlord().State()
@@ -932,7 +932,7 @@ func (s *userSuite) TestPostCreateUserAutomaticDisabled(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/create-user", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 
 	// empty result
 	expected := []daemon.UserResponseData{}
@@ -974,7 +974,7 @@ func (s *userSuite) TestPostCreateUserExpirationHappy(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/create-user", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 	c.Check(rsp.Result, check.DeepEquals, &daemon.UserResponseData{
 		Username: expectedUsername,
 		SSHKeys: []string{
@@ -991,7 +991,7 @@ func (s *userSuite) TestPostCreateUserExpirationDateSetInPast(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/create-user", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Message, check.Matches, `cannot create user: expiration date must be set in the future`)
 }
 
@@ -1001,7 +1001,7 @@ func (s *userSuite) TestPostCreateUserExpirationKnownNotAllowed(c *check.C) {
 	req, err := http.NewRequest("POST", "/v2/create-user", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Message, check.Matches, `cannot create user: expiration date cannot be provided for known users`)
 }
 
@@ -1013,7 +1013,7 @@ func (s *userSuite) TestPostCreateUserExpirationAutomaticNotAllowed(c *check.C)
 	req, err := http.NewRequest("POST", "/v2/create-user", buf)
 	c.Assert(err, check.IsNil)
 
-	rspe := s.errorReq(c, req, nil)
+	rspe := s.errorReq(c, req, nil, actionIsExpected)
 	c.Check(rspe.Message, check.Matches, `cannot create user: expiration date cannot be provided for known users`)
 }
 
@@ -1021,7 +1021,7 @@ func (s *userSuite) TestUsersEmpty(c *check.C) {
 	req, err := http.NewRequest("GET", "/v2/users", nil)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 
 	expected := []daemon.UserResponseData{}
 	c.Check(rsp.Result, check.FitsTypeOf, expected)
@@ -1043,7 +1043,7 @@ func (s *userSuite) TestUsersHasUser(c *check.C) {
 	req, err := http.NewRequest("GET", "/v2/users", nil)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 
 	expected := []daemon.UserResponseData{
 		{ID: u.ID, Username: u.Username, Email: u.Email},
@@ -1053,7 +1053,7 @@ func (s *userSuite) TestUsersHasUser(c *check.C) {
 }
 
 func (s *userSuite) testPostCreateUserFromAssertion(c *check.C, postData string, expectSudoer bool) {
-	s.makeSystemUsers(c, []map[string]interface{}{goodUser, partnerUser, serialUser, badUser, badUserNoMatchingSerial, unknownUser})
+	s.makeSystemUsers(c, []map[string]any{goodUser, partnerUser, serialUser, badUser, badUserNoMatchingSerial, unknownUser})
 
 	// mock the calls that create the user
 	var deviceStateCreateUserCalled bool
@@ -1082,7 +1082,7 @@ func (s *userSuite) testPostCreateUserFromAssertion(c *check.C, postData string,
 	req, err := http.NewRequest("POST", "/v2/create-user", buf)
 	c.Assert(err, check.IsNil)
 
-	rsp := s.syncReq(c, req, nil)
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
 
 	// note that we get a list here instead of a single
 	// userResponseData item
