@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2020 Canonical Ltd
+ * Copyright (C) 2016-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -17,13 +17,13 @@
  *
  */
 
-// Package apparmor implements integration between snappy and
-// ubuntu-core-launcher around apparmor.
+// Package apparmor implements integration between snappy and snap-confine
+// around apparmor.
 //
 // Snappy creates apparmor profiles for each application (for each snap)
-// present in the system.  Upon each execution of ubuntu-core-launcher
-// application process is launched under the profile. Prior to that the profile
-// must be parsed, compiled and loaded into the kernel using the support tool
+// present in the system.  Upon each execution of snap-confine application
+// process is launched under the profile. Prior to that the profile must be
+// parsed, compiled and loaded into the kernel using the support tool
 // "apparmor_parser".
 //
 // Each apparmor profile contains a simple <header><content><footer> structure.
@@ -54,6 +54,7 @@ import (
 	"github.com/snapcore/snapd/release"
 	apparmor_sandbox "github.com/snapcore/snapd/sandbox/apparmor"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snapdenv"
 	"github.com/snapcore/snapd/strutil"
 	"github.com/snapcore/snapd/timings"
 )
@@ -68,7 +69,8 @@ var (
 
 	// make sure that apparmor profile fulfills the late discarding backend
 	// interface
-	_ interfaces.SecurityBackendDiscardingLate = (*Backend)(nil)
+	_ interfaces.SecurityBackendDiscardingLate  = (*Backend)(nil)
+	_ interfaces.ReinitializableSecurityBackend = (*Backend)(nil)
 )
 
 // Backend is responsible for maintaining apparmor profiles for snaps and parts of snapd.
@@ -94,6 +96,15 @@ func (b *Backend) Initialize(opts *interfaces.SecurityBackendOptions) error {
 		b.coreSnap = opts.CoreSnapInfo
 		b.snapdSnap = opts.SnapdSnapInfo
 	}
+
+	const reinitializing = false
+	if err := b.initializeSnapConfineProfiles(b.preseed, reinitializing); err != nil {
+		return fmt.Errorf("cannot initialize snap-confine profiles: %w", err)
+	}
+	return nil
+}
+
+func (b *Backend) initializeSnapConfineProfiles(preseed, reinitializing bool) error {
 	// NOTE: It would be nice if we could also generate the profile for
 	// snap-confine executing from the core snap, right here, and not have to
 	// do this in the Setup function below. I sadly don't think this is
@@ -104,22 +115,34 @@ func (b *Backend) Initialize(opts *interfaces.SecurityBackendOptions) error {
 	// fail early if this fails for whatever reason.
 	exe, err := os.Readlink(procSelfExe)
 	if err != nil {
-		return fmt.Errorf("cannot read %s: %s", procSelfExe, err)
+		return fmt.Errorf("cannot read %s: %w", procSelfExe, err)
 	}
 
-	if _, err := apparmor_sandbox.SetupSnapConfineSnippets(); err != nil {
+	// TODO:mismatch:optimization: regenerate a subset of snippets if needed?
+
+	// snippets are shared between all snap-confine profiles, those from
+	// snapd/core snap and the distro one
+	changed, err := apparmor_sandbox.SetupSnapConfineSnippets()
+	if err != nil {
 		return err
 	}
 
-	// If snapd is executing from the core snap the it means it has
-	// re-executed. In that case we are no longer using the copy of
-	// snap-confine from the host distribution but our own copy. We don't have
-	// to re-compile and load the updated profile as that is performed by
-	// setupSnapConfineReexec below.
-	if strings.HasPrefix(exe, dirs.SnapMountDir) {
+	if reinitializing && !changed {
+		logger.Noticef("snap-confine snippets not changed during reinit, not reloading profiles")
 		return nil
 	}
 
+	// If snapd is executing from the snapd or core snap the it means it has
+	// re-executed. In that case we are no longer using the copy of snap-confine
+	// from the host distribution but our own copy. We don't have to re-compile
+	// and load the updated profile as that will be done when preparing security
+	// profiles of the snapd/core snap.
+	if strings.HasPrefix(exe, dirs.SnapMountDir) {
+		logger.Notice("delegating reload of snap-confine profiles to system snap security profiles setup")
+		return nil
+	}
+
+	logger.Notice("reloading profiles for snap-confine")
 	// Reload the apparmor profile of snap-confine. This points to the main
 	// file in /etc/apparmor.d/ as that file contains include statements that
 	// load any of the files placed in /var/lib/snapd/apparmor/snap-confine/.
@@ -136,7 +159,7 @@ func (b *Backend) Initialize(opts *interfaces.SecurityBackendOptions) error {
 	}
 
 	aaFlags := apparmor_sandbox.SkipReadCache
-	if b.preseed {
+	if preseed {
 		aaFlags |= apparmor_sandbox.SkipKernelLoad
 	}
 
@@ -145,7 +168,17 @@ func (b *Backend) Initialize(opts *interfaces.SecurityBackendOptions) error {
 		// policy. Maybe we have caused the problem so it's better to let other
 		// things work.
 		apparmor_sandbox.RemoveSnapConfineSnippets()
-		return fmt.Errorf("cannot reload snap-confine apparmor profile: %v", err)
+		return fmt.Errorf("cannot reload snap-confine apparmor profile: %w", err)
+	}
+	return nil
+}
+
+// Reinitialize performs a limited reinitialization of the backend. Implements
+// interfaces.ReinitializableSecurityBackend.
+func (b *Backend) Reinitialize() error {
+	const reinitializing = true
+	if err := b.initializeSnapConfineProfiles(b.preseed, reinitializing); err != nil {
+		return fmt.Errorf("cannot initialize snap-confine profiles: %w", err)
 	}
 	return nil
 }
@@ -249,6 +282,8 @@ func snapConfineProfileName(snapName string, rev snap.Revision) string {
 //
 // Additionally it will cleanup stale apparmor profiles it created.
 func (b *Backend) setupSnapConfineReexec(info *snap.Info) error {
+	logger.Noticef("reloading profiles of snap-confine provided by the system snap")
+
 	if err := os.MkdirAll(apparmor_sandbox.SnapConfineAppArmorDir, 0755); err != nil {
 		return fmt.Errorf("cannot create snap-confine policy directory: %s", err)
 	}
@@ -309,7 +344,7 @@ func nsProfile(snapName string) string {
 // apps and hooks while the second profile describes the snap-update-ns profile
 // for the whole snap.
 func profileGlobs(snapName string) []string {
-	return []string{interfaces.SecurityTagGlob(snapName), nsProfile(snapName)}
+	return append(interfaces.SecurityTagGlobs(snapName), nsProfile(snapName))
 }
 
 // Determine if a profile filename is removable during core refresh/rollback.
@@ -345,7 +380,7 @@ type profilePathsResults struct {
 
 func (b *Backend) prepareProfiles(appSet *interfaces.SnapAppSet, opts interfaces.ConfinementOptions, repo *interfaces.Repository) (prof *profilePathsResults, err error) {
 	snapName := appSet.InstanceName()
-	spec, err := repo.SnapSpecification(b.Name(), appSet)
+	spec, err := repo.SnapSpecification(b.Name(), appSet, opts)
 	if err != nil {
 		return nil, fmt.Errorf("cannot obtain apparmor specification for snap %q: %s", snapName, err)
 	}
@@ -356,7 +391,7 @@ func (b *Backend) prepareProfiles(appSet *interfaces.SnapAppSet, opts interfaces
 	spec.(*Specification).AddOvername(snapInfo)
 
 	// Add snippets derived from the layout definition.
-	spec.(*Specification).AddLayout(snapInfo)
+	spec.(*Specification).AddLayout(appSet)
 
 	// Add additional mount layouts rules for the snap.
 	spec.(*Specification).AddExtraLayouts(snapInfo, opts.ExtraLayouts)
@@ -395,13 +430,15 @@ func (b *Backend) prepareProfiles(appSet *interfaces.SnapAppSet, opts interfaces
 	}
 
 	// Get the files that this snap should have
-	content := b.deriveContent(spec.(*Specification), snapInfo, opts)
+	content := b.deriveContent(spec.(*Specification), appSet, opts)
 
 	dir := dirs.SnapAppArmorDir
-	globs := profileGlobs(snapInfo.InstanceName())
 	if err := os.MkdirAll(dir, 0755); err != nil {
 		return nil, fmt.Errorf("cannot create directory for apparmor profiles %q: %s", dir, err)
 	}
+
+	globs := profileGlobs(snapInfo.InstanceName())
+
 	changed, removedPaths, errEnsure := osutil.EnsureDirStateGlobs(dir, globs, content)
 	// XXX: in the old code this error was reported late, after doing load/removeCached.
 	if errEnsure != nil {
@@ -612,21 +649,15 @@ var (
 	coreRuntimePattern = regexp.MustCompile("^core([0-9][0-9])?$")
 )
 
-func (b *Backend) deriveContent(spec *Specification, snapInfo *snap.Info, opts interfaces.ConfinementOptions) (content map[string]osutil.FileState) {
-	content = make(map[string]osutil.FileState, len(snapInfo.Apps)+len(snapInfo.Hooks)+1)
+func (b *Backend) deriveContent(spec *Specification, appSet *interfaces.SnapAppSet, opts interfaces.ConfinementOptions) (content map[string]osutil.FileState) {
+	runnables := appSet.Runnables()
+	content = make(map[string]osutil.FileState, len(runnables))
+	snapInfo := appSet.Info()
 
-	// Add profile for each app.
-	for _, appInfo := range snapInfo.Apps {
-		securityTag := appInfo.SecurityTag()
-		b.addContent(securityTag, snapInfo, appInfo.Name, opts, spec.SnippetForTag(securityTag), content, spec)
+	// Add profile for apps and hooks.
+	for _, r := range runnables {
+		b.addContent(r.SecurityTag, snapInfo, r.CommandName, opts, spec.SnippetForTag(r.SecurityTag), content, spec)
 	}
-	// Add profile for each hook.
-	for _, hookInfo := range snapInfo.Hooks {
-		securityTag := hookInfo.SecurityTag()
-		b.addContent(securityTag, snapInfo, "hook."+hookInfo.Name, opts, spec.SnippetForTag(securityTag), content, spec)
-	}
-
-	// TODO: something with component hooks will need to happen here
 
 	// Add profile for snap-update-ns if we have any apps or hooks.
 	// If we have neither then we don't have any need to create an executing environment.
@@ -664,7 +695,9 @@ func addUpdateNSProfile(snapInfo *snap.Info, snippets string, content map[string
 			}
 			return ""
 		default:
-			// TODO: Warn that an invalid pattern is being used.
+			if snapdenv.Testing() || osutil.IsTestBinary() {
+				panic(fmt.Sprintf("cannot expand snippet for pattern %q", placeholder))
+			}
 		}
 		return ""
 	})
@@ -677,6 +710,9 @@ func addUpdateNSProfile(snapInfo *snap.Info, snippets string, content map[string
 	}
 }
 
+// Allow optional trailing ' ' after "###PROMPT###"
+var promptReplacer = regexp.MustCompile("###PROMPT### ?")
+
 func (b *Backend) addContent(securityTag string, snapInfo *snap.Info, cmdName string, opts interfaces.ConfinementOptions, snippetForTag string, content map[string]osutil.FileState, spec *Specification) {
 	// If base is specified and it doesn't match the core snaps (not
 	// specifying a base should use the default core policy since in this
@@ -698,6 +734,15 @@ func (b *Backend) addContent(securityTag string, snapInfo *snap.Info, cmdName st
 	}
 	policy = templatePattern.ReplaceAllStringFunc(policy, func(placeholder string) string {
 		switch placeholder {
+		case "###KERNEL_MODULES_AND_FIRMWARE###":
+			if opts.KernelSnap != "" {
+				return fmt.Sprintf(`
+  # Allow access to kernel modules and firmware from the kernel snap
+  /snap/%[1]s/*/{modules,firmware}/{,**} r,
+  /snap/%[1]s/components/mnt/*/*/{modules,firmware}/{,**} r,
+  /var/snap/%[1]s/*/{modules,firmware}/{,**} r,`, opts.KernelSnap)
+			}
+			return ""
 		case "###DEVMODE_SNAP_CONFINE###":
 			if !opts.DevMode {
 				// nothing to add if we are not in devmode
@@ -959,6 +1004,13 @@ func (b *Backend) addContent(securityTag string, snapInfo *snap.Info, cmdName st
 				}
 				tagSnippets = strings.Replace(tagSnippets, "###HOME_IX###", repl, -1)
 
+				// Use prompt prefix if prompting is supported and enabled
+				repl = ""
+				if spec.UsePromptPrefix() {
+					repl = "prompt "
+				}
+				tagSnippets = promptReplacer.ReplaceAllLiteralString(tagSnippets, repl)
+
 				// Conditionally add privilege dropping policy
 				if len(snapInfo.SystemUsernames) > 0 {
 					tagSnippets += privDropAndChownRules
@@ -966,6 +1018,12 @@ func (b *Backend) addContent(securityTag string, snapInfo *snap.Info, cmdName st
 			}
 
 			return tagSnippets
+		default:
+			if snapdenv.Testing() || osutil.IsTestBinary() {
+				panic(fmt.Sprintf("cannot expand snippet for pattern %q", placeholder))
+			} else {
+				logger.Noticef("WARNING: cannto expand snippet for pattern %q", placeholder)
+			}
 		}
 		return ""
 	})
@@ -977,8 +1035,11 @@ func (b *Backend) addContent(securityTag string, snapInfo *snap.Info, cmdName st
 }
 
 // NewSpecification returns a new, empty apparmor specification.
-func (b *Backend) NewSpecification(appSet *interfaces.SnapAppSet) interfaces.Specification {
-	return &Specification{appSet: appSet}
+func (b *Backend) NewSpecification(appSet *interfaces.SnapAppSet, opts interfaces.ConfinementOptions) interfaces.Specification {
+	return &Specification{
+		appSet:          appSet,
+		usePromptPrefix: opts.AppArmorPrompting,
+	}
 }
 
 // SandboxFeatures returns the list of apparmor features supported by the kernel.
