@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2018 Canonical Ltd
+ * Copyright (C) 2016-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -28,6 +28,7 @@ import (
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/builtin"
+	"github.com/snapcore/snapd/interfaces/ifacetest"
 	"github.com/snapcore/snapd/interfaces/policy"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
@@ -67,11 +68,13 @@ plugs:
   %s:
 `, iface)
 	}
-	slotSnap := snaptest.MockInfo(c, slotYaml, nil)
-	plugSnap := snaptest.MockInfo(c, plugYaml, nil)
+
+	slot, _ := ifacetest.MockConnectedSlot(c, slotYaml, nil, iface)
+	plug, _ := ifacetest.MockConnectedPlug(c, plugYaml, nil, iface)
+
 	return &policy.ConnectCandidate{
-		Plug:            interfaces.NewConnectedPlug(plugSnap.Plugs[iface], nil, nil),
-		Slot:            interfaces.NewConnectedSlot(slotSnap.Slots[iface], nil, nil),
+		Plug:            plug,
+		Slot:            slot,
 		BaseDeclaration: s.baseDecl,
 	}
 }
@@ -140,23 +143,29 @@ func (s *baseDeclSuite) TestAutoConnection(c *C) {
 	// these have more complex or in flux policies and have their
 	// own separate tests
 	snowflakes := map[string]bool{
-		"content":            true,
-		"core-support":       true,
-		"home":               true,
-		"lxd-support":        true,
-		"microstack-support": true,
-		"multipass-support":  true,
-		"packagekit-control": true,
-		"pkcs11":             true,
-		"snapd-control":      true,
-		"empty":              true,
+		"content":                true,
+		"core-support":           true,
+		"cuda-driver-libs":       true,
+		"desktop":                true,
+		"egl-driver-libs":        true,
+		"home":                   true,
+		"lxd-support":            true,
+		"microstack-support":     true,
+		"multipass-support":      true,
+		"packagekit-control":     true,
+		"pkcs11":                 true,
+		"remoteproc":             true,
+		"screen-inhibit-control": true,
+		"snapd-control":          true,
+		"upower-observe":         true,
+		"empty":                  true,
 	}
 
 	// these simply auto-connect, anything else doesn't
 	autoconnect := map[string]bool{
+		"accel":                   true,
 		"audio-playback":          true,
 		"browser-support":         true,
-		"desktop":                 true,
 		"desktop-legacy":          true,
 		"gsettings":               true,
 		"media-hub":               true,
@@ -167,11 +176,10 @@ func (s *baseDeclSuite) TestAutoConnection(c *C) {
 		"online-accounts-service": true,
 		"opengl":                  true,
 		"optical-drive":           true,
-		"screen-inhibit-control":  true,
+		"ros-opt-data":            true,
 		"ubuntu-download-manager": true,
 		"unity7":                  true,
 		"unity8":                  true,
-		"upower-observe":          true,
 		"wayland":                 true,
 		"x11":                     true,
 	}
@@ -195,6 +203,35 @@ func (s *baseDeclSuite) TestAutoConnection(c *C) {
 	}
 }
 
+func (s *baseDeclSuite) TestAutoConnectionImplicitSlotOnly(c *C) {
+	all := builtin.Interfaces()
+
+	// these auto-connect only with an implicit slot
+	autoconnect := map[string]bool{
+		"desktop":                true,
+		"screen-inhibit-control": true,
+		"upower-observe":         true,
+	}
+
+	for _, iface := range all {
+		if !autoconnect[iface.Name()] {
+			continue
+		}
+		comm := Commentf(iface.Name())
+
+		// check base declaration
+		cand := s.connectCand(c, iface.Name(), fmt.Sprintf(`name: snapd
+type: snapd
+version: 0
+slots:
+  %s:
+`, iface.Name()), "")
+		arity, err := cand.CheckAutoConnect()
+		c.Check(err, IsNil, comm)
+		c.Check(arity.SlotsPerPlugAny(), Equals, false)
+	}
+}
+
 func (s *baseDeclSuite) TestAutoConnectPlugSlot(c *C) {
 	all := builtin.Interfaces()
 
@@ -203,6 +240,7 @@ func (s *baseDeclSuite) TestAutoConnectPlugSlot(c *C) {
 	snowflakes := map[string]bool{
 		"classic-support": true,
 		"content":         true,
+		"cups-control":    true,
 		"home":            true,
 		"lxd-support":     true,
 		// netlink-driver needs the family-name attributes to match
@@ -773,7 +811,8 @@ var (
 		"bluez":                     {"app", "core"},
 		"bool-file":                 {"core", "gadget"},
 		"browser-support":           {"core"},
-		"content":                   {"app", "gadget"},
+		"checkbox-support":          {"core"},
+		"content":                   {"app", "gadget", "kernel"},
 		"core-support":              {"core"},
 		"cups":                      {"app"},
 		"cups-control":              {"app", "core"},
@@ -809,14 +848,17 @@ var (
 		"online-accounts-service":   {"app"},
 		"power-control":             {"core"},
 		"ppp":                       {"core"},
+		"polkit-agent":              {"core"},
+		"pipewire":                  {"app", "core"},
 		"pulseaudio":                {"app", "core"},
 		"pwm":                       {"core", "gadget"},
-		"qualcomm-ipc-router":       {"core"},
+		"qualcomm-ipc-router":       {"core", "app"},
 		"raw-volume":                {"core", "gadget"},
 		"scsi-generic":              {"core"},
 		"sd-control":                {"core"},
 		"serial-port":               {"core", "gadget"},
 		"spi":                       {"core", "gadget"},
+		"screen-inhibit-control":    {"core", "app"},
 		"steam-support":             {"core"},
 		"storage-framework-service": {"app"},
 		"thumbnailer-service":       {"app"},
@@ -828,18 +870,23 @@ var (
 		"unity8-calendar":           {"app"},
 		"unity8-contacts":           {"app"},
 		"upower-observe":            {"app", "core"},
+		"userns":                    {"core"},
 		"wayland":                   {"app", "core"},
 		"x11":                       {"app", "core"},
 		// snowflakes
-		"classic-support": nil,
-		"custom-device":   nil,
-		"docker":          nil,
-		"lxd":             nil,
-		"microceph":       nil,
-		"microovn":        nil,
-		"pkcs11":          nil,
-		"posix-mq":        nil,
-		"shared-memory":   nil,
+		"classic-support":   nil,
+		"cuda-driver-libs":  nil,
+		"custom-device":     nil,
+		"docker":            nil,
+		"egl-driver-libs":   nil,
+		"lxd":               nil,
+		"microceph":         nil,
+		"microceph-support": nil,
+		"microovn":          nil,
+		"pkcs11":            nil,
+		"posix-mq":          nil,
+		"shared-memory":     nil,
+		"gpio-chardev":      nil,
 	}
 
 	restrictedPlugInstallation = map[string][]string{
@@ -863,6 +910,10 @@ func (s *baseDeclSuite) TestSlotInstallation(c *C) {
 			types = []string{"core"}
 		}
 
+		// only restricted slots can use the AppArmor
+		// unconfined profile mode so check that this
+		// slot is not using it
+		c.Assert(interfaces.StaticInfoOf(iface).AppArmorUnconfinedSlots, Equals, false)
 		if types == nil {
 			// snowflake needs to be tested specially
 			continue
@@ -880,9 +931,22 @@ func (s *baseDeclSuite) TestSlotInstallation(c *C) {
 		}
 	}
 
-	// test docker specially
-	ic := s.installSlotCand(c, "docker", snap.TypeApp, ``)
+	// test desktop specifically
+	ic := s.installSlotCand(c, "desktop", snap.TypeApp, ``)
 	err := ic.Check()
+	c.Check(err, Not(IsNil))
+	c.Check(err, ErrorMatches, "installation denied by \"desktop\" slot rule of interface \"desktop\"")
+	// ... but the minimal check (used by --dangerous) allows installation
+	icMin := &policy.InstallCandidateMinimalCheck{
+		Snap:            ic.Snap,
+		BaseDeclaration: s.baseDecl,
+	}
+	err = icMin.Check()
+	c.Check(err, IsNil)
+
+	// test docker specially
+	ic = s.installSlotCand(c, "docker", snap.TypeApp, ``)
+	err = ic.Check()
 	c.Assert(err, Not(IsNil))
 	c.Assert(err, ErrorMatches, "installation not allowed by \"docker\" slot rule of interface \"docker\"")
 
@@ -910,6 +974,25 @@ func (s *baseDeclSuite) TestSlotInstallation(c *C) {
 	c.Assert(err, Not(IsNil))
 	c.Assert(err, ErrorMatches, "installation denied by \"shared-memory\" slot rule of interface \"shared-memory\"")
 
+	// test gpio-chardev specifically as it needs correct attributes in the snap yaml
+	for name, snapType := range snapTypeMap {
+		ic := s.installSlotCand(c, "gpio-chardev", snapType, fmt.Sprintf(`name: my-device
+version: 0
+type: %s
+slots:
+  gpio-chardev:
+    source-chip: [chip0]
+    lines: 0,1
+`, snapType))
+		err := ic.Check()
+		comm := Commentf("%s by %s snap", "gpio-chardev", name)
+		if snapType == snap.TypeGadget {
+			c.Check(err, IsNil, comm)
+		} else {
+			c.Check(err, NotNil, comm)
+		}
+	}
+
 	// The core and snapd snaps may provide a shared-memory slot
 	ic = s.installSlotCand(c, "shared-memory", snap.TypeOS, `name: core
 version: 0
@@ -954,35 +1037,52 @@ func (s *baseDeclSuite) TestPlugInstallation(c *C) {
 	all := builtin.Interfaces()
 
 	restricted := map[string]bool{
-		"block-devices":          true,
-		"classic-support":        true,
-		"desktop-launch":         true,
-		"dm-crypt":               true,
-		"docker-support":         true,
-		"greengrass-support":     true,
-		"gpio-control":           true,
-		"ion-memory-control":     true,
-		"kernel-module-control":  true,
-		"kernel-module-load":     true,
-		"kubernetes-support":     true,
-		"lxd-support":            true,
-		"microstack-support":     true,
-		"mount-control":          true,
-		"multipass-support":      true,
-		"nvidia-drivers-support": true,
-		"packagekit-control":     true,
-		"personal-files":         true,
-		"polkit":                 true,
-		"sd-control":             true,
-		"snap-refresh-control":   true,
-		"snap-themes-control":    true,
-		"snapd-control":          true,
-		"steam-support":          true,
-		"system-files":           true,
-		"tee":                    true,
-		"uinput":                 true,
-		"unity8":                 true,
-		"xilinx-dma":             true,
+		"auditd-support":                   true,
+		"block-devices":                    true,
+		"checkbox-support":                 true,
+		"classic-support":                  true,
+		"cuda-driver-libs":                 true,
+		"desktop-launch":                   true,
+		"dm-crypt":                         true,
+		"docker-support":                   true,
+		"egl-driver-libs":                  true,
+		"greengrass-support":               true,
+		"gpio-control":                     true,
+		"ion-memory-control":               true,
+		"kernel-firmware-control":          true,
+		"kernel-module-control":            true,
+		"kernel-module-load":               true,
+		"kubernetes-support":               true,
+		"lxd-support":                      true,
+		"microceph-support":                true,
+		"microstack-support":               true,
+		"mount-control":                    true,
+		"multipass-support":                true,
+		"nomad-support":                    true,
+		"nvidia-drivers-support":           true,
+		"packagekit-control":               true,
+		"personal-files":                   true,
+		"polkit":                           true,
+		"polkit-agent":                     true,
+		"remoteproc":                       true,
+		"ros-snapd-support":                true,
+		"scsi-generic":                     true,
+		"sd-control":                       true,
+		"shutdown":                         true,
+		"snap-interfaces-requests-control": true,
+		"snap-refresh-control":             true,
+		"snap-refresh-observe":             true,
+		"snap-themes-control":              true,
+		"snapd-control":                    true,
+		"steam-support":                    true,
+		"system-files":                     true,
+		"tee":                              true,
+		"uinput":                           true,
+		"unity8":                           true,
+		"userns":                           true,
+		"xilinx-dma":                       true,
+		"snap-fde-control":                 true,
+		"firmware-updater-support":         true,
 	}
 
 	for _, iface := range all {
@@ -991,6 +1091,10 @@ func (s *baseDeclSuite) TestPlugInstallation(c *C) {
 		// need to make sure this is really the case here. If that is not
 		// the case we continue as normal.
 		if ok {
+			// only restricted plugs can use the AppArmor
+			// unconfined profile mode so check that this
+			// plug is not using it
+			c.Assert(interfaces.StaticInfoOf(iface).AppArmorUnconfinedPlugs, Equals, false)
 			for name, snapType := range snapTypeMap {
 				ok := strutil.ListContains(types, name)
 				ic := s.installPlugCand(c, iface.Name(), snapType, ``)
@@ -1010,6 +1114,10 @@ func (s *baseDeclSuite) TestPlugInstallation(c *C) {
 				c.Check(err, NotNil, comm)
 			} else {
 				c.Check(err, IsNil, comm)
+				// only restricted plugs can use the AppArmor
+				// unconfined profile mode so check that this
+				// plug is not using it
+				c.Assert(interfaces.StaticInfoOf(iface).AppArmorUnconfinedPlugs, Equals, false)
 			}
 		}
 	}
@@ -1024,6 +1132,7 @@ func (s *baseDeclSuite) TestConnection(c *C) {
 		"content":                   true,
 		"cups":                      true,
 		"custom-device":             true,
+		"desktop":                   true,
 		"docker":                    true,
 		"fwupd":                     true,
 		"location-control":          true,
@@ -1035,13 +1144,16 @@ func (s *baseDeclSuite) TestConnection(c *C) {
 		"mir":                       true,
 		"online-accounts-service":   true,
 		"posix-mq":                  true,
+		"qualcomm-ipc-router":       true,
 		"raw-volume":                true,
+		"screen-inhibit-control":    true,
 		"shared-memory":             true,
 		"storage-framework-service": true,
 		"thumbnailer-service":       true,
 		"ubuntu-download-manager":   true,
 		"unity8-calendar":           true,
 		"unity8-contacts":           true,
+		"upower-observe":            true,
 	}
 
 	for _, iface := range all {
@@ -1060,6 +1172,35 @@ func (s *baseDeclSuite) TestConnection(c *C) {
 	}
 }
 
+func (s *baseDeclSuite) TestConnectionImplicitSlotOnly(c *C) {
+	all := builtin.Interfaces()
+
+	// these allow connect only with an implicit slot
+	autoconnect := map[string]bool{
+		"desktop":                true,
+		"qualcomm-ipc-router":    true,
+		"screen-inhibit-control": true,
+		"upower-observe":         true,
+	}
+
+	for _, iface := range all {
+		if !autoconnect[iface.Name()] {
+			continue
+		}
+		comm := Commentf(iface.Name())
+
+		// check base declaration
+		cand := s.connectCand(c, iface.Name(), fmt.Sprintf(`name: snapd
+type: snapd
+version: 0
+slots:
+  %s:
+`, iface.Name()), "")
+		err := cand.Check()
+		c.Check(err, IsNil, comm)
+	}
+}
+
 func (s *baseDeclSuite) TestConnectionOnClassic(c *C) {
 	restore := release.MockOnClassic(false)
 	defer restore()
@@ -1074,7 +1215,6 @@ func (s *baseDeclSuite) TestConnectionOnClassic(c *C) {
 		"network-manager": true,
 		"ofono":           true,
 		"pulseaudio":      true,
-		"upower-observe":  true,
 	}
 
 	for _, onClassic := range []bool{true, false} {
@@ -1130,7 +1270,6 @@ func (s *baseDeclSuite) TestConnectionImplicitOnClassicOrAppSnap(c *C) {
 		"network-manager": true,
 		"ofono":           true,
 		"pulseaudio":      true,
-		"upower-observe":  true,
 	}
 
 	for _, iface := range all {
@@ -1203,43 +1342,64 @@ func (s *baseDeclSuite) TestValidity(c *C) {
 	// given how the rules work this can be delicate,
 	// listed here to make sure that was a conscious decision
 	bothSides := map[string]bool{
-		"block-devices":          true,
-		"audio-playback":         true,
-		"classic-support":        true,
-		"core-support":           true,
-		"custom-device":          true,
-		"desktop-launch":         true,
-		"dm-crypt":               true,
-		"docker-support":         true,
-		"greengrass-support":     true,
-		"gpio-control":           true,
-		"ion-memory-control":     true,
-		"kernel-module-control":  true,
-		"kernel-module-load":     true,
-		"kubernetes-support":     true,
-		"lxd-support":            true,
-		"microstack-support":     true,
-		"mount-control":          true,
-		"multipass-support":      true,
-		"nvidia-drivers-support": true,
-		"packagekit-control":     true,
-		"personal-files":         true,
-		"pkcs11":                 true,
-		"posix-mq":               true,
-		"polkit":                 true,
-		"sd-control":             true,
-		"shared-memory":          true,
-		"snap-refresh-control":   true,
-		"snap-themes-control":    true,
-		"snapd-control":          true,
-		"steam-support":          true,
-		"system-files":           true,
-		"tee":                    true,
-		"udisks2":                true,
-		"uinput":                 true,
-		"unity8":                 true,
-		"wayland":                true,
-		"xilinx-dma":             true,
+		"auditd-support":                   true,
+		"block-devices":                    true,
+		"audio-playback":                   true,
+		"classic-support":                  true,
+		"checkbox-support":                 true,
+		"core-support":                     true,
+		"cuda-driver-libs":                 true,
+		"custom-device":                    true,
+		"desktop":                          true,
+		"desktop-launch":                   true,
+		"dm-crypt":                         true,
+		"docker-support":                   true,
+		"egl-driver-libs":                  true,
+		"greengrass-support":               true,
+		"gpio-control":                     true,
+		"ion-memory-control":               true,
+		"kernel-firmware-control":          true,
+		"kernel-module-control":            true,
+		"kernel-module-load":               true,
+		"kubernetes-support":               true,
+		"lxd-support":                      true,
+		"microceph-support":                true,
+		"microstack-support":               true,
+		"mount-control":                    true,
+		"multipass-support":                true,
+		"nomad-support":                    true,
+		"nvidia-drivers-support":           true,
+		"packagekit-control":               true,
+		"personal-files":                   true,
+		"pkcs11":                           true,
+		"posix-mq":                         true,
+		"polkit":                           true,
+		"polkit-agent":                     true,
+		"remoteproc":                       true,
+		"ros-snapd-support":                true,
+		"qualcomm-ipc-router":              true,
+		"scsi-generic":                     true,
+		"screen-inhibit-control":           true,
+		"sd-control":                       true,
+		"shutdown":                         true,
+		"shared-memory":                    true,
+		"snap-interfaces-requests-control": true,
+		"snap-refresh-control":             true,
+		"snap-refresh-observe":             true,
+		"snap-themes-control":              true,
+		"snapd-control":                    true,
+		"steam-support":                    true,
+		"system-files":                     true,
+		"tee":                              true,
+		"udisks2":                          true,
+		"uinput":                           true,
+		"unity8":                           true,
+		"userns":                           true,
+		"wayland":                          true,
+		"xilinx-dma":                       true,
+		"confdb":                           true,
+		"snap-fde-control":                 true,
+		"firmware-updater-support":         true,
 	}
 
 	for _, iface := range all {
@@ -1318,6 +1478,72 @@ plugs:
 	c.Check(err, NotNil)
 }
 
+func (s *baseDeclSuite) TestConnectionQualcommIpcRouter(c *C) {
+	// we let connect explicitly as long as qcipc matches
+
+	slotDecl1 := s.mockSnapDecl(c, "slot-snap", "slot-snap-id", "pub1", "")
+	plugDecl1 := s.mockSnapDecl(c, "plug-snap", "plug-snap-id", "pub1", "")
+
+	// Same qcipc label
+	cand := s.connectCand(c, "qc-router", `name: slot-snap
+version: 0
+slots:
+  qc-router:
+    interface: qualcomm-ipc-router
+    qcipc: monitor
+    address: abcd
+`, `
+name: plug-snap
+version: 0
+plugs:
+  qc-router:
+    interface: qualcomm-ipc-router
+    qcipc: monitor
+`)
+	cand.SlotSnapDeclaration = slotDecl1
+	cand.PlugSnapDeclaration = plugDecl1
+	err := cand.Check()
+	c.Check(err, IsNil)
+
+	// Different qcipc label
+	cand = s.connectCand(c, "qc-router", `name: slot-snap
+version: 0
+slots:
+  qc-router:
+    interface: qualcomm-ipc-router
+    qcipc: monitor
+    address: abcd
+`, `
+name: plug-snap
+version: 0
+plugs:
+  qc-router:
+    interface: qualcomm-ipc-router
+    qcipc: other
+`)
+	cand.SlotSnapDeclaration = slotDecl1
+	cand.PlugSnapDeclaration = plugDecl1
+	err = cand.Check()
+	c.Check(err.Error(), Equals, `connection not allowed by slot rule of interface "qualcomm-ipc-router"`)
+
+	// Legacy case with slot provided by system
+	cand = s.connectCand(c, "qualcomm-ipc-router", `name: snapd
+version: 0
+type: snapd
+slots:
+  qualcomm-ipc-router:
+`, `
+name: plug-snap
+version: 0
+plugs:
+  qualcomm-ipc-router:
+`)
+	cand.SlotSnapDeclaration = s.mockSnapDecl(c, "snapd", "PMrrV4ml8uWuEUDBT8dSGnKUYbevVhc4", "canonical", "")
+	cand.PlugSnapDeclaration = plugDecl1
+	err = cand.Check()
+	c.Check(err, IsNil)
+}
+
 func (s *baseDeclSuite) TestConnectionSharedMemory(c *C) {
 	// we let connect explicitly as long as shared-memory matches
 
@@ -1530,28 +1756,28 @@ plugs:
 }
 
 func (s *baseDeclSuite) TestRawVolumeOverride(c *C) {
-	slotYaml := `name: slot-snap
+	const slotYaml = `name: slot-snap
 type: gadget
 version: 0
 slots:
   raw-volume:
     path: /dev/mmcblk0p1
 `
-	slotSnap := snaptest.MockInfo(c, slotYaml, nil)
+	slot, _ := ifacetest.MockConnectedSlot(c, slotYaml, nil, "raw-volume")
 	// mock a well-formed slot snap decl with SnapID
 	slotSnapDecl := s.mockSnapDecl(c, "slot-snap", "slotsnapidididididididididididid", "canonical", "")
 
-	plugYaml := `name: plug-snap
+	const plugYaml = `name: plug-snap
 version: 0
 plugs:
   raw-volume:
 `
-	plugSnap := snaptest.MockInfo(c, plugYaml, nil)
+	plug, _ := ifacetest.MockConnectedPlug(c, plugYaml, nil, "raw-volume")
 
 	// no plug-side declaration
 	cand := &policy.ConnectCandidate{
-		Plug:                interfaces.NewConnectedPlug(plugSnap.Plugs["raw-volume"], nil, nil),
-		Slot:                interfaces.NewConnectedSlot(slotSnap.Slots["raw-volume"], nil, nil),
+		Plug:                plug,
+		Slot:                slot,
 		SlotSnapDeclaration: slotSnapDecl,
 		BaseDeclaration:     s.baseDecl,
 	}
@@ -1632,3 +1858,109 @@ plugs:
 	_, err = cand.CheckAutoConnect()
 	c.Check(err, IsNil)
 }
+
+func (s *baseDeclSuite) TestAutoConnectionPolkitAgentOverride(c *C) {
+	cand := s.connectCand(c, "polkit-agent", "", "")
+	_, err := cand.CheckAutoConnect()
+	c.Check(err, NotNil)
+	c.Assert(err, ErrorMatches, "auto-connection denied by plug rule of interface \"polkit-agent\"")
+
+	plugsSlots := `
+plugs:
+  polkit-agent:
+    allow-auto-connection: true
+`
+
+	snapDecl := s.mockSnapDecl(c, "some-snap", "some-snap-with-polkit-agent-id", "canonical", plugsSlots)
+	cand.PlugSnapDeclaration = snapDecl
+	_, err = cand.CheckAutoConnect()
+	c.Check(err, IsNil)
+}
+
+func (s *baseDeclSuite) TestConnectionDesktop(c *C) {
+	cand := s.connectCand(c, "desktop", "", "")
+	err := cand.Check()
+	c.Assert(err, ErrorMatches, `connection denied by plug rule of interface "desktop"`)
+
+	plugsSlots := `
+plugs:
+  desktop:
+    allow-connection: true
+`
+	snapDecl := s.mockSnapDecl(c, "some-snap", "some-snap-with-desktop-id", "canonical", plugsSlots)
+	cand.PlugSnapDeclaration = snapDecl
+	err = cand.Check()
+	c.Assert(err, IsNil)
+}
+
+func (s *baseDeclSuite) TestAutoConnectionDesktop(c *C) {
+	cand := s.connectCand(c, "desktop", "", "")
+	_, err := cand.CheckAutoConnect()
+	c.Assert(err, ErrorMatches, "auto-connection denied by plug rule of interface \"desktop\"")
+
+	plugsSlots := `
+plugs:
+  desktop:
+    allow-auto-connection: true
+`
+	snapDecl := s.mockSnapDecl(c, "some-snap", "some-snap-with-desktop-id", "canonical", plugsSlots)
+	cand.PlugSnapDeclaration = snapDecl
+	_, err = cand.CheckAutoConnect()
+	c.Check(err, IsNil)
+}
+
+func (s *baseDeclSuite) TestDesktopFileIDsOverride(c *C) {
+	ic := s.installPlugCand(c, "desktop", snap.TypeApp, `name: some-snap
+version: 0
+type: app
+plugs:
+  desktop:
+    desktop-file-ids: [org.example]
+`)
+	err := ic.Check()
+	c.Assert(err, ErrorMatches, `installation not allowed by "desktop" plug rule of interface "desktop"`)
+
+	const plugsOverride = `
+plugs:
+  desktop:
+    allow-installation:
+      plug-attributes:
+        desktop-file-ids:
+          - org.example
+`
+	ic.SnapDeclaration = s.mockSnapDecl(c, "some-snap", "some-snap-id", "canonical", plugsOverride)
+	err = ic.Check()
+	c.Assert(err, IsNil)
+}
+
+func (s *baseDeclSuite) TestConnectionScreenInhibitControl(c *C) {
+	cand := s.connectCand(c, "screen-inhibit-control", "", "")
+	err := cand.Check()
+	c.Assert(err, ErrorMatches, `connection denied by slot rule of interface "screen-inhibit-control"`)
+
+	plugsSlots := `
+plugs:
+  screen-inhibit-control:
+    allow-connection: true
+`
+	snapDecl := s.mockSnapDecl(c, "some-snap", "some-snap", "canonical", plugsSlots)
+	cand.PlugSnapDeclaration = snapDecl
+	err = cand.Check()
+	c.Assert(err, IsNil)
+}
+
+func (s *baseDeclSuite) TestAutoConnectionScreenInhibitControl(c *C) {
+	cand := s.connectCand(c, "screen-inhibit-control", "", "")
+	_, err := cand.CheckAutoConnect()
+	c.Assert(err, ErrorMatches, "auto-connection denied by slot rule of interface \"screen-inhibit-control\"")
+
+	plugsSlots := `
+plugs:
+  screen-inhibit-control:
+    allow-auto-connection: true
+`
+	snapDecl := s.mockSnapDecl(c, "some-snap", "some-snap", "canonical", plugsSlots)
+	cand.PlugSnapDeclaration = snapDecl
+	_, err = cand.CheckAutoConnect()
+	c.Check(err, IsNil)
+}
