@@ -1,6 +1,5 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 //go:build arm || 386
-// +build arm 386
 
 /*
  * Copyright (C) 2017 Canonical Ltd
@@ -27,8 +26,10 @@ import "syscall"
 // (and so the 32 bit version is called getuid32 etc)
 
 const (
-	_SYS_GETUID  = syscall.SYS_GETUID32
-	_SYS_GETGID  = syscall.SYS_GETGID32
-	_SYS_GETEUID = syscall.SYS_GETEUID32
-	_SYS_GETEGID = syscall.SYS_GETEGID32
+	_SYS_GETUID   = syscall.SYS_GETUID32
+	_SYS_GETGID   = syscall.SYS_GETGID32
+	_SYS_GETEUID  = syscall.SYS_GETEUID32
+	_SYS_GETEGID  = syscall.SYS_GETEGID32
+	_SYS_SETREUID = syscall.SYS_SETREUID32
+	_SYS_SETREGID = syscall.SYS_SETREGID32
 )
