@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2020 Canonical Ltd
+ * Copyright (C) 2020-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -28,6 +28,7 @@ import (
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/asserts/snapasserts"
+	"github.com/snapcore/snapd/confdb"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/release"
@@ -141,6 +142,26 @@ func bulkRefreshSnapDeclarations(s *state.State, snapStates map[string]*snapstat
 	return nil
 }
 
+func bulkRefreshConfdbSchemas(s *state.State, confdbSchemas []*confdb.Schema, userID int, deviceCtx snapstate.DeviceContext, opts *RefreshAssertionsOptions) error {
+	db := cachedDB(s)
+
+	// all assertion refs will be in the same group
+	pool := asserts.NewPool(db, maxGroups)
+	for _, confdb := range confdbSchemas {
+		account, name := confdb.Account, confdb.Name
+		ref := &asserts.Ref{
+			Type:       asserts.ConfdbSchemaType,
+			PrimaryKey: []string{account, name},
+		}
+
+		if err := pool.AddToUpdate(ref, storeGroup); err != nil {
+			return fmt.Errorf("cannot prepare confdb assertion %s/%s for refresh: %v", account, name, err)
+		}
+	}
+
+	return resolvePool(s, pool, nil, userID, deviceCtx, opts)
+}
+
 func bulkRefreshValidationSetAsserts(s *state.State, vsets map[string]*ValidationSetTracking, beforeCommitChecker func(*asserts.Database, asserts.Backstore) error, userID int, deviceCtx snapstate.DeviceContext, opts *RefreshAssertionsOptions) error {
 	db := cachedDB(s)
 	pool := asserts.NewPool(db, maxGroups)
