@@ -107,7 +107,7 @@ func (s *gateAutoRefreshHookSuite) TearDownTest(c *C) {
 	s.commonTearDownTest(c)
 }
 
-func mockRefreshCandidate(snapName, instanceKey, channel, version string, revision snap.Revision) interface{} {
+func mockRefreshCandidate(snapName, instanceKey, channel, version string, revision snap.Revision) any {
 	sup := &snapstate.SnapSetup{
 		Channel:     channel,
 		InstanceKey: instanceKey,
@@ -126,13 +126,13 @@ func (s *gateAutoRefreshHookSuite) settle(c *C) {
 }
 
 func checkIsHeld(c *C, st *state.State, heldSnap, gatingSnap string) {
-	var held map[string]map[string]interface{}
+	var held map[string]map[string]any
 	c.Assert(st.Get("snaps-hold", &held), IsNil)
 	c.Check(held[heldSnap][gatingSnap], NotNil)
 }
 
 func checkIsNotHeld(c *C, st *state.State, heldSnap string) {
-	var held map[string]map[string]interface{}
+	var held map[string]map[string]any
 	c.Assert(st.Get("snaps-hold", &held), IsNil)
 	c.Check(held[heldSnap], IsNil)
 }
@@ -145,7 +145,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookProceedRuninhibitLock(
 		defer ctx.Unlock()
 
 		// check that runinhibit hint has been set by Before() hook handler.
-		hint, info, err := runinhibit.IsLocked("snap-a")
+		hint, info, err := runinhibit.IsLocked("snap-a", nil)
 		c.Assert(err, IsNil)
 		c.Check(hint, Equals, runinhibit.HintInhibitedGateRefresh)
 		c.Check(info, Equals, runinhibit.InhibitInfo{Previous: snap.R(1)})
@@ -178,7 +178,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookProceedRuninhibitLock(
 	c.Assert(change.Err(), IsNil)
 	c.Assert(change.Status(), Equals, state.DoneStatus)
 
-	hint, info, err := runinhibit.IsLocked("snap-a")
+	hint, info, err := runinhibit.IsLocked("snap-a", nil)
 	c.Assert(err, IsNil)
 	c.Check(hint, Equals, runinhibit.HintInhibitedForRefresh)
 	c.Check(info, Equals, runinhibit.InhibitInfo{Previous: snap.R(1)})
@@ -192,7 +192,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookHoldUnlocksRuninhibit(
 		defer ctx.Unlock()
 
 		// check that runinhibit hint has been set by Before() hook handler.
-		hint, info, err := runinhibit.IsLocked("snap-a")
+		hint, info, err := runinhibit.IsLocked("snap-a", nil)
 		c.Assert(err, IsNil)
 		c.Check(hint, Equals, runinhibit.HintInhibitedGateRefresh)
 		c.Check(info, Equals, runinhibit.InhibitInfo{Previous: snap.R(1)})
@@ -226,7 +226,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookHoldUnlocksRuninhibit(
 	c.Assert(change.Status(), Equals, state.DoneStatus)
 
 	// runinhibit lock is released.
-	hint, info, err := runinhibit.IsLocked("snap-a")
+	hint, info, err := runinhibit.IsLocked("snap-a", nil)
 	c.Assert(err, IsNil)
 	c.Check(hint, Equals, runinhibit.HintNotInhibited)
 	c.Check(info, Equals, runinhibit.InhibitInfo{})
@@ -237,7 +237,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookHoldUnlocksRuninhibit(
 func (s *gateAutoRefreshHookSuite) TestGateAutorefreshDefaultProceedUnlocksRuninhibit(c *C) {
 	hookInvoke := func(ctx *hookstate.Context, tomb *tomb.Tomb) ([]byte, error) {
 		// validity, refresh is inhibited for snap-a.
-		hint, info, err := runinhibit.IsLocked("snap-a")
+		hint, info, err := runinhibit.IsLocked("snap-a", nil)
 		c.Assert(err, IsNil)
 		c.Check(hint, Equals, runinhibit.HintInhibitedGateRefresh)
 		c.Check(info, Equals, runinhibit.InhibitInfo{Previous: snap.R(1)})
@@ -279,7 +279,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshDefaultProceedUnlocksRunin
 	checkIsNotHeld(c, st, "snap-a")
 
 	// runinhibit lock is released.
-	hint, info, err := runinhibit.IsLocked("snap-a")
+	hint, info, err := runinhibit.IsLocked("snap-a", nil)
 	c.Assert(err, IsNil)
 	c.Check(hint, Equals, runinhibit.HintNotInhibited)
 	c.Check(info, Equals, runinhibit.InhibitInfo{})
@@ -290,7 +290,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshDefaultProceedUnlocksRunin
 func (s *gateAutoRefreshHookSuite) TestGateAutorefreshDefaultProceed(c *C) {
 	hookInvoke := func(ctx *hookstate.Context, tomb *tomb.Tomb) ([]byte, error) {
 		// no runinhibit because the refresh-app-awareness feature is disabled.
-		hint, info, err := runinhibit.IsLocked("snap-a")
+		hint, info, err := runinhibit.IsLocked("snap-a", nil)
 		c.Assert(err, IsNil)
 		c.Check(hint, Equals, runinhibit.HintNotInhibited)
 		c.Check(info, Equals, runinhibit.InhibitInfo{})
@@ -327,7 +327,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshDefaultProceed(c *C) {
 	checkIsNotHeld(c, st, "snap-b")
 
 	// no runinhibit because the refresh-app-awareness feature is disabled.
-	hint, info, err := runinhibit.IsLocked("snap-a")
+	hint, info, err := runinhibit.IsLocked("snap-a", nil)
 	c.Assert(err, IsNil)
 	c.Check(hint, Equals, runinhibit.HintNotInhibited)
 	c.Check(info, Equals, runinhibit.InhibitInfo{})
@@ -338,7 +338,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshDefaultProceed(c *C) {
 func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookError(c *C) {
 	hookInvoke := func(ctx *hookstate.Context, tomb *tomb.Tomb) ([]byte, error) {
 		// no runinhibit because the refresh-app-awareness feature is disabled.
-		hint, info, err := runinhibit.IsLocked("snap-a")
+		hint, info, err := runinhibit.IsLocked("snap-a", nil)
 		c.Assert(err, IsNil)
 		c.Check(hint, Equals, runinhibit.HintNotInhibited)
 		c.Check(info, Equals, runinhibit.InhibitInfo{})
@@ -355,7 +355,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookError(c *C) {
 	st.Lock()
 	defer st.Unlock()
 
-	candidates := map[string]interface{}{"snap-a": mockRefreshCandidate("snap-a", "", "edge", "v1", snap.Revision{N: 3})}
+	candidates := map[string]any{"snap-a": mockRefreshCandidate("snap-a", "", "edge", "v1", snap.Revision{N: 3})}
 	st.Set("refresh-candidates", candidates)
 
 	task := hookstate.SetupGateAutoRefreshHook(st, "snap-a")
@@ -373,7 +373,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookError(c *C) {
 	checkIsHeld(c, st, "snap-a", "snap-a")
 
 	// no runinhibit because the refresh-app-awareness feature is disabled.
-	hint, info, err := runinhibit.IsLocked("snap-a")
+	hint, info, err := runinhibit.IsLocked("snap-a", nil)
 	c.Assert(err, IsNil)
 	c.Check(hint, Equals, runinhibit.HintNotInhibited)
 	c.Check(info, Equals, runinhibit.InhibitInfo{})
@@ -384,7 +384,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookError(c *C) {
 func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorAfterProceed(c *C) {
 	hookInvoke := func(ctx *hookstate.Context, tomb *tomb.Tomb) ([]byte, error) {
 		// no runinhibit because the refresh-app-awareness feature is disabled.
-		hint, info, err := runinhibit.IsLocked("snap-a")
+		hint, info, err := runinhibit.IsLocked("snap-a", nil)
 		c.Assert(err, IsNil)
 		c.Check(hint, Equals, runinhibit.HintNotInhibited)
 		c.Check(info, Equals, runinhibit.InhibitInfo{})
@@ -407,7 +407,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorAfterProceed(c *C
 	st.Lock()
 	defer st.Unlock()
 
-	candidates := map[string]interface{}{"snap-a": mockRefreshCandidate("snap-a", "", "edge", "v1", snap.Revision{N: 3})}
+	candidates := map[string]any{"snap-a": mockRefreshCandidate("snap-a", "", "edge", "v1", snap.Revision{N: 3})}
 	st.Set("refresh-candidates", candidates)
 
 	task := hookstate.SetupGateAutoRefreshHook(st, "snap-a")
@@ -425,7 +425,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorAfterProceed(c *C
 	checkIsHeld(c, st, "snap-a", "snap-a")
 
 	// no runinhibit because the refresh-app-awareness feature is disabled.
-	hint, info, err := runinhibit.IsLocked("snap-a")
+	hint, info, err := runinhibit.IsLocked("snap-a", nil)
 	c.Assert(err, IsNil)
 	c.Check(hint, Equals, runinhibit.HintNotInhibited)
 	c.Check(info, Equals, runinhibit.InhibitInfo{})
@@ -436,7 +436,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorAfterProceed(c *C
 func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorRuninhibitUnlock(c *C) {
 	hookInvoke := func(ctx *hookstate.Context, tomb *tomb.Tomb) ([]byte, error) {
 		// no runinhibit because the refresh-app-awareness feature is disabled.
-		hint, info, err := runinhibit.IsLocked("snap-a")
+		hint, info, err := runinhibit.IsLocked("snap-a", nil)
 		c.Assert(err, IsNil)
 		c.Check(hint, Equals, runinhibit.HintInhibitedGateRefresh)
 		c.Check(info, Equals, runinhibit.InhibitInfo{Previous: snap.R(1)})
@@ -458,7 +458,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorRuninhibitUnlock(
 	tr.Set("core", "experimental.refresh-app-awareness", true)
 	tr.Commit()
 
-	candidates := map[string]interface{}{"snap-a": mockRefreshCandidate("snap-a", "", "edge", "v1", snap.Revision{N: 3})}
+	candidates := map[string]any{"snap-a": mockRefreshCandidate("snap-a", "", "edge", "v1", snap.Revision{N: 3})}
 	st.Set("refresh-candidates", candidates)
 
 	task := hookstate.SetupGateAutoRefreshHook(st, "snap-a")
@@ -476,7 +476,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorRuninhibitUnlock(
 	checkIsHeld(c, st, "snap-a", "snap-a")
 
 	// inhibit lock is unlocked
-	hint, info, err := runinhibit.IsLocked("snap-a")
+	hint, info, err := runinhibit.IsLocked("snap-a", nil)
 	c.Assert(err, IsNil)
 	c.Check(hint, Equals, runinhibit.HintNotInhibited)
 	c.Check(info, Equals, runinhibit.InhibitInfo{})
@@ -485,7 +485,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorRuninhibitUnlock(
 func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorHoldErrorLogged(c *C) {
 	hookInvoke := func(ctx *hookstate.Context, tomb *tomb.Tomb) ([]byte, error) {
 		// no runinhibit because the refresh-app-awareness feature is disabled.
-		hint, info, err := runinhibit.IsLocked("snap-a")
+		hint, info, err := runinhibit.IsLocked("snap-a", nil)
 		c.Assert(err, IsNil)
 		c.Check(hint, Equals, runinhibit.HintNotInhibited)
 		c.Check(info, Equals, runinhibit.InhibitInfo{})
@@ -504,7 +504,7 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorHoldErrorLogged(c
 	st.Lock()
 	defer st.Unlock()
 
-	candidates := map[string]interface{}{"snap-a": mockRefreshCandidate("snap-a", "", "edge", "v1", snap.Revision{N: 3})}
+	candidates := map[string]any{"snap-a": mockRefreshCandidate("snap-a", "", "edge", "v1", snap.Revision{N: 3})}
 	st.Set("refresh-candidates", candidates)
 
 	task := hookstate.SetupGateAutoRefreshHook(st, "snap-a")
@@ -527,12 +527,12 @@ func (s *gateAutoRefreshHookSuite) TestGateAutorefreshHookErrorHoldErrorLogged(c
 	c.Assert(change.Status(), Equals, state.DoneStatus)
 
 	// and snap-b is not held (due to hold error).
-	var held map[string]map[string]interface{}
+	var held map[string]map[string]any
 	c.Assert(st.Get("snaps-hold", &held), IsNil)
 	c.Check(held, HasLen, 0)
 
 	// no runinhibit because the refresh-app-awareness feature is disabled.
-	hint, info, err := runinhibit.IsLocked("snap-a")
+	hint, info, err := runinhibit.IsLocked("snap-a", nil)
 	c.Assert(err, IsNil)
 	c.Check(hint, Equals, runinhibit.HintNotInhibited)
 	c.Check(info, Equals, runinhibit.InhibitInfo{})
