@@ -31,15 +31,19 @@ import (
 	"github.com/snapcore/snapd/i18n"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/policy"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord/assertstate"
 	"github.com/snapcore/snapd/overlord/hookstate"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
+	"github.com/snapcore/snapd/overlord/swfeats"
 	"github.com/snapcore/snapd/snap"
 )
 
 var connectRetryTimeout = time.Second * 5
 
+var regenerateSecurityProfilesChangeKind = swfeats.RegisterChangeKind("regenerate-security-profiles")
+
 // ErrAlreadyConnected describes the error that occurs when attempting to connect already connected interface.
 type ErrAlreadyConnected struct {
 	Connection interfaces.ConnRef
@@ -146,7 +150,7 @@ func connect(st *state.State, plugSnap, plugName, slotSnap, slotName string, fla
 	}
 
 	connectInterface := st.NewTask("connect", fmt.Sprintf(i18n.G("Connect %s:%s to %s:%s"), plugSnap, plugName, slotSnap, slotName))
-	initialContext := make(map[string]interface{})
+	initialContext := make(map[string]any)
 	initialContext["attrs-task"] = connectInterface.ID()
 
 	tasks := state.NewTaskSet()
@@ -211,7 +215,7 @@ func connect(st *state.State, plugSnap, plugName, slotSnap, slotName string, fla
 
 	// Expose a copy of all plug and slot attributes coming from yaml to interface hooks. The hooks will be able
 	// to modify them but all attributes will be checked against assertions after the hooks are run.
-	emptyDynamicAttrs := map[string]interface{}{}
+	emptyDynamicAttrs := map[string]any{}
 	connectInterface.Set("plug-static", plugStatic)
 	connectInterface.Set("slot-static", slotStatic)
 	connectInterface.Set("plug-dynamic", emptyDynamicAttrs)
@@ -280,13 +284,17 @@ func connect(st *state.State, plugSnap, plugName, slotSnap, slotName string, fla
 	return tasks, nil
 }
 
-func initialConnectAttributes(st *state.State, plugSnapInfo *snap.Info, plugSnap string, plugName string, slotSnapInfo *snap.Info, slotSnap string, slotName string) (plugStatic, slotStatic map[string]interface{}, err error) {
+func initialConnectAttributes(st *state.State, plugSnapInfo *snap.Info, plugSnap string, plugName string, slotSnapInfo *snap.Info, slotSnap string, slotName string) (plugStatic, slotStatic map[string]any, err error) {
 	var plugSnapst snapstate.SnapState
 
 	if err = snapstate.Get(st, plugSnap, &plugSnapst); err != nil {
 		return nil, nil, err
 	}
 
+	if err := addImplicitInterfaces(st, plugSnapInfo); err != nil {
+		return nil, nil, err
+	}
+
 	plug, ok := plugSnapInfo.Plugs[plugName]
 	if !ok {
 		return nil, nil, fmt.Errorf("snap %q has no plug named %q", plugSnap, plugName)
@@ -298,7 +306,7 @@ func initialConnectAttributes(st *state.State, plugSnapInfo *snap.Info, plugSnap
 		return nil, nil, err
 	}
 
-	if err := addImplicitSlots(st, slotSnapInfo); err != nil {
+	if err := addImplicitInterfaces(st, slotSnapInfo); err != nil {
 		return nil, nil, err
 	}
 
@@ -408,7 +416,7 @@ func disconnectTasks(st *state.State, conn *interfaces.Connection, flags disconn
 		prev = t
 	}
 
-	initialContext := make(map[string]interface{})
+	initialContext := make(map[string]any)
 	initialContext["attrs-task"] = disconnectTask.ID()
 
 	plugSnapInfo, err := plugSnapst.CurrentInfo()
@@ -475,7 +483,7 @@ func disconnectTasks(st *state.State, conn *interfaces.Connection, flags disconn
 // CheckInterfaces checks whether plugs and slots of snap are allowed for installation.
 func CheckInterfaces(st *state.State, snapInfo *snap.Info, deviceCtx snapstate.DeviceContext) error {
 	// XXX: addImplicitSlots is really a brittle interface
-	if err := addImplicitSlots(st, snapInfo); err != nil {
+	if err := addImplicitInterfaces(st, snapInfo); err != nil {
 		return err
 	}
 
@@ -570,9 +578,100 @@ func OnSnapLinkageChanged(st *state.State, snapsup *snapstate.SnapSetup) error {
 		// track the revision that was just unlinked that has
 		// still profiles
 		snapst.PendingSecurity = &snapstate.PendingSecurityState{
-			SideInfo: snapst.CurrentSideInfo(),
+			SideInfo:   snapst.CurrentSideInfo(),
+			Components: snapst.CurrentComponentSideInfos(),
 		}
 	}
 	snapstate.Set(st, instanceName, &snapst)
 	return nil
 }
+
+// InterfacesRequestsControlHandlerServices returns the list of all apps which
+// are defined as "handler-service" for a snap which has a connected plug for
+// the "snap-interfaces-requests-control" interface.
+//
+// The caller must ensure that the given state is locked.
+func InterfacesRequestsControlHandlerServices(st *state.State) ([]*snap.AppInfo, error) {
+	conns, err := ConnectionStates(st)
+	if err != nil {
+		return nil, fmt.Errorf("internal error: cannot get connections: %w", err)
+	}
+
+	var handlers []*snap.AppInfo
+
+	for connId, connState := range conns {
+		if connState.Interface != "snap-interfaces-requests-control" || !connState.Active() {
+			continue
+		}
+
+		connRef, err := interfaces.ParseConnRef(connId)
+		if err != nil {
+			return nil, err
+		}
+
+		handler, ok := connState.StaticPlugAttrs["handler-service"].(string)
+		if !ok {
+			// does not have a handler service
+			continue
+		}
+
+		sn := connRef.PlugRef.Snap
+		si, err := snapstate.CurrentInfo(st, sn)
+		if err != nil {
+			return nil, err
+		}
+
+		// this should not fail as the plug's BeforePrepare should have validated that such an app exists
+		app := si.Apps[handler]
+		if app == nil {
+			return nil, fmt.Errorf("internal error: cannot find app %q in snap %q", app, sn)
+		}
+
+		handlers = append(handlers, app)
+	}
+
+	return handlers, nil
+}
+
+// AdviseReportedSystemKeyMismatch inspects the system key, which is reportedly
+// in a mismatch with the recoded one, and decides to either create a state
+// change for regenerating security profiles, thus returning a change, or do
+// nothing, in which case no change is returned.
+func AdviseReportedSystemKeyMismatch(st *state.State, systemKey any) (*state.Change, error) {
+	var seeded bool
+	err := st.Get("seeded", &seeded)
+	if err != nil && !errors.Is(err, state.ErrNoState) {
+		return nil, err
+	}
+
+	if !seeded {
+		// System not ready yet for checking system-key, bubble up the error to
+		// clients. They can either wait or proceed at their own peril.
+		return nil, errors.New("system not yet seeded")
+	}
+
+	action, err := interfaces.SystemKeyMismatchAdvice(systemKey)
+	if err != nil {
+		return nil, err
+	}
+
+	logger.Debugf("reported system key mismatch action: %v", action)
+
+	if action == interfaces.SystemKeyMismatchActionNone {
+		// nothing to do
+		return nil, nil
+	}
+
+	for _, chg := range st.Changes() {
+		// if we have a change that isn't ready, return it instead
+		if chg.Kind() == regenerateSecurityProfilesChangeKind && !chg.IsReady() {
+			return chg, nil
+		}
+	}
+
+	chg := st.NewChange(regenerateSecurityProfilesChangeKind, "Regenerate security profiles")
+	t := st.NewTask(regenerateSecurityProfilesChangeKind, "Regenerate security profiles")
+	chg.AddTask(t)
+
+	return chg, nil
+}
